# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import pytest

from otx.api.usecases.adapters.model_adapter import (
    ExportableCodeAdapter,
    IDataSource,
    ModelAdapter,
)
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestIDataSource:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_i_data_source_data(self):
        """
        <b>Description:</b>
        Check IDataSource class "data" property

        <b>Input data:</b>
        IDataSource object

        <b>Expected results:</b>
        Test passes if IDataSource object "data" property raises NotImplementedError exception
        """
        with pytest.raises(NotImplementedError):
            IDataSource().data()


class DummyDataSource(IDataSource):
    def __init__(self, data: str):
        self._data = data

    @property
    def data(self):
        return self._data


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestModelAdapter:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_model_adapter_initialization(self):
        """
        <b>Description:</b>
        Check ModelAdapter object initialization

        <b>Input data:</b>
        ModelAdapter object initialized with "data_source" IDataSource or bytes parameter

        <b>Expected results:</b>
        Test passes if properties of initialized ModelAdapter object are equal to expected

        <b>Steps</b>
        1. Check attributes of ModelAdapter object initialized with IDataSource "data_source" parameter
        2. Check attributes of ModelAdapter object initialized with bytes "data_source" parameter
        3. Check that ValueError exception is raised when initializing ModelAdapter object with "data_source" parameter
        type not equal to bytes or IDataSource
        """
        # Checking properties of "ModelAdapter" initialized with IDataSource "data_source"
        data = "some data"
        data_source = DummyDataSource(data=data)
        model_adapter = ModelAdapter(data_source=data_source)
        assert model_adapter.data_source == data_source
        assert model_adapter.from_file_storage
        assert model_adapter.data == data
        # Checking properties of "ModelAdapter" initialized with bytes "data_source"
        data_source = b"binaryrepo://localhost/repo/data_source/1"
        model_adapter = ModelAdapter(data_source=data_source)
        assert model_adapter.data_source == data_source
        assert not model_adapter.from_file_storage
        assert model_adapter.data == data_source
        # Checking that ValueError exception is raised when initializing ModelAdapter with "data_source" type not equal
        # to bytes or IDataSource
        model_adapter = ModelAdapter(data_source=1)  # type: ignore
        with pytest.raises(ValueError):
            model_adapter.data()

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_model_adapter_data_source_setter(self):
        """
        <b>Description:</b>
        Check ModelAdapter "data_source" property setter

        <b>Input data:</b>
        ModelAdapter object initialized with "data_source" parameter

        <b>Expected results:</b>
        Test passes if properties of ModelAdapter are equal to expected after manual setting "data_source" property

        <b>Steps</b>
        1. Check properties of ModelAdapter object after manual setting "data_source" property to other IDataSource
        object
        2. Check properties of ModelAdapter object after manual setting "data_source" property to bytes object
        3. Check properties of ModelAdapter object after manual setting "data_source" property to other bytes object
        4. Check properties of ModelAdapter object after manual setting "data_source" property to IDataSource object
        """
        model_adapter = ModelAdapter(data_source=DummyDataSource(data="some data"))
        # Checking properties of ModelAdapter after manual setting "data_source" to other IDataSource
        other_data = "other data"
        other_data_source = DummyDataSource(data=other_data)
        model_adapter.data_source = other_data_source
        assert model_adapter.data_source == other_data_source
        assert model_adapter.data == other_data
        assert model_adapter.from_file_storage
        # Checking properties of ModelAdapter after manual setting "data_source" to bytes
        bytes_data_source = b"binaryrepo://localhost/repo/data_source/1"
        model_adapter.data_source = bytes_data_source
        assert model_adapter.data_source == bytes_data_source
        assert model_adapter.data == bytes_data_source
        assert not model_adapter.from_file_storage
        # Checking properties of ModelAdapter after manual setting "data_source" to other bytes
        other_bytes_data_source = b"binaryrepo://localhost/repo/data_source/2"
        model_adapter.data_source = b"binaryrepo://localhost/repo/data_source/2"
        assert model_adapter.data_source == other_bytes_data_source
        assert model_adapter.data == other_bytes_data_source
        assert not model_adapter.from_file_storage
        # Checking properties of ModelAdapter after manual setting "data_source" to IDataSource
        model_adapter.data_source = other_data_source
        assert model_adapter.data_source == other_data_source
        assert model_adapter.data == other_data
        assert model_adapter.from_file_storage


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestExportableCodeAdapter:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_exportable_code_adapter_initialization(self):
        """
        <b>Description:</b>
        Check ExportableCodeAdapter object initialization

        <b>Input data:</b>
        ExportableCodeAdapter object initialized with "data_source" IDataSource or bytes parameter

        <b>Expected results:</b>
        Test passes if properties of initialized ExportableCodeAdapter object are equal to expected

        <b>Steps</b>
        1. Check attributes of ExportableCodeAdapter object initialized with IDataSource "data_source" parameter
        2. Check attributes of ExportableCodeAdapter object initialized with bytes "data_source" parameter
        3. Check that ValueError exception is raised when initializing ExportableCodeAdapter object with "data_source"
        parameter type not equal to bytes or IDataSource
        """
        # Checking properties of "ExportableCodeAdapter" initialized with IDataSource "data_source"
        data = "some_data"
        data_source = DummyDataSource(data=data)
        exportable_code_adapter = ExportableCodeAdapter(data_source=data_source)
        assert exportable_code_adapter.data_source == data_source
        assert exportable_code_adapter.from_file_storage
        assert exportable_code_adapter.data == data
        # Checking properties after setting "data_source" to bytes
        bytes_data_source = b"binaryrepo://localhost/repo/data_source/1"
        exportable_code_adapter.data_source = bytes_data_source
        assert exportable_code_adapter.data_source == bytes_data_source
        assert not exportable_code_adapter.from_file_storage
        assert exportable_code_adapter.data == bytes_data_source
        # Checking properties of "ExportableCodeAdapter" initialized with bytes "data_source"
        exportable_code_adapter = ExportableCodeAdapter(data_source=bytes_data_source)
        assert exportable_code_adapter.data_source == bytes_data_source
        assert not exportable_code_adapter.from_file_storage
        assert exportable_code_adapter.data == bytes_data_source
        # Checking properties after setting "data_source" to IDataSource
        exportable_code_adapter.data_source = data_source
        assert exportable_code_adapter.data_source == data_source
        assert exportable_code_adapter.from_file_storage
        assert exportable_code_adapter.data == data
        # Checking that ValueError exception is raised when initializing ExportableCodeAdapter with "data_source" type
        # not equal to bytes or IDataSource
        exportable_code_adapter = ExportableCodeAdapter(data_source=1)  # type: ignore
        with pytest.raises(ValueError):
            exportable_code_adapter.data()



"""MobileNet-V3-Small for multi-class config."""

# pylint: disable=invalid-name

_base_ = ["../../../../recipes/stages/classification/incremental.yaml", "../base/models/mobilenet_v3.py"]

model = dict(
    type="SAMImageClassifier",
    task="classification",
)



"""Unit test for otx.algorithms.common.adapters.mmcv.hooks.recording_forward_hook."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import numpy as np
import pytest
import torch

from otx.algorithms.common.adapters.mmcv.hooks.recording_forward_hook import (
    ActivationMapHook,
    BaseRecordingForwardHook,
    EigenCamHook,
    FeatureVectorHook,
    ReciproCAMHook,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class MockBaseRecordingForwardHook(BaseRecordingForwardHook):
    @staticmethod
    def func(*args):
        return torch.Tensor([[0]])


class TestBaseRecordingForwardHook:
    """Test class for BaseRecordingForwardHook"""

    @e2e_pytest_unit
    def test_records(self, mocker) -> None:
        """Test records function."""

        hook = MockBaseRecordingForwardHook(torch.nn.Module())
        assert hook.records == []

    @e2e_pytest_unit
    def test_func(self) -> None:
        """Test func function."""
        hook = MockBaseRecordingForwardHook(torch.nn.Module())
        assert hook.func() == torch.Tensor([[0]])

    @e2e_pytest_unit
    def test_recording_forward(self) -> None:
        """Test _recording_forward."""

        hook = MockBaseRecordingForwardHook(torch.nn.Module())
        hook._recording_forward(torch.nn.Module(), torch.Tensor([0]), torch.Tensor([0]))
        assert hook._records == [np.array([0.0])]

    @e2e_pytest_unit
    def test_enter(self) -> None:
        """Test __enter__ function."""

        class _MockModule(torch.nn.Module):
            def __init__(self) -> None:
                super().__init__()
                self.backbone = torch.nn.Module()

        hook = MockBaseRecordingForwardHook(_MockModule())
        hook.__enter__()

    @e2e_pytest_unit
    def test_exit(self) -> None:
        """Test __exit__ function."""

        class MockHandle:
            def remove(self):
                pass

        hook = MockBaseRecordingForwardHook(torch.nn.Module())
        hook._handle = MockHandle()
        hook.__exit__(None, None, None)


class TestEigenCamHook:
    """Test class for EigenCamHook."""

    def test_func(self) -> None:
        """Test func function."""

        hook = EigenCamHook(torch.nn.Module())
        feature_map = torch.randn(8, 3, 14, 14)
        assert hook.func(feature_map) is not None


class TestActivationMapHook:
    """Test class for ActivationMapHook."""

    def test_func(self) -> None:
        """Test func function."""

        hook = ActivationMapHook(torch.nn.Module())
        feature_map = torch.randn(8, 3, 14, 14)
        assert hook.func(feature_map) is not None


class TestFeatureVectorHook:
    """Test class for FeatureVectorHook."""

    def test_func(self) -> None:
        """Test func function."""

        hook = FeatureVectorHook(torch.nn.Module())
        feature_map = torch.randn(8, 3, 14, 14)
        assert hook.func(feature_map) is not None


class TestReciproCAMHook:
    """Test class for ReciproCAMHook."""

    @pytest.fixture(autouse=True)
    def setup(self) -> None:
        class _MockModule(torch.nn.Module):
            def __init__(self) -> None:
                super().__init__()
                self.with_neck = True
                self.neck = torch.nn.Module()
                self.neck.forward = self.forward
                self.head = torch.nn.Module()
                self.head.num_classes = 3
                self.head.simple_test = self.forward

            def forward(self, x):
                return x

        self.module = _MockModule()
        self.hook = ReciproCAMHook(self.module)

    @e2e_pytest_unit
    def test_func(self) -> None:
        """Test func function."""

        assert self.hook.func([torch.randn(1, 3, 1, 1)]) is not None



"""Adapters of classification - mmcls."""

# Copyright (C) 2022 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.


# The import required to register the backbone used by the OTX Template with the Registry.
import otx.algorithms.common.adapters.mmcv.models as OTXBackbones

from .datasets import OTXClsDataset, SelfSLDataset
from .models import BYOL, ConstrastiveHead, SelfSLMLP
from .optimizer import LARS

# fmt: off
# isort: off
# FIXME: openvino pot library adds stream handlers to root logger
# which makes annoying duplicated logging
from mmcls.utils import get_root_logger  # pylint: disable=wrong-import-order
get_root_logger().propagate = False
# isort:on
# fmt: on

__all__ = ["OTXClsDataset", "SelfSLDataset", "BYOL", "SelfSLMLP", "ConstrastiveHead", "LARS", "OTXBackbones"]



"""Module to init transforms for OTX classification."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#
# flake8: noqa

from .augmix import AugMixAugment
from .otx_transforms import PILToTensor, RandomRotate, TensorNormalize
from .random_augment import OTXRandAugment
from .twocrop_transform import TwoCropTransform

__all__ = [
    "AugMixAugment",
    "PILToTensor",
    "TensorNormalize",
    "RandomRotate",
    "OTXRandAugment",
    "TwoCropTransform",
]



# Copyright (C) 2021-2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import numpy as np
import torch

from otx.algorithms.common.adapters.mmdeploy.utils.utils import (
    numpy_2_list,
    sync_batchnorm_2_batchnorm,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


def create_model():
    class MockModel(torch.nn.Module):
        def __init__(self, *args, **kwargs):
            super().__init__()
            self.conv1 = torch.nn.Conv2d(3, 3, 3)
            self.norm = torch.nn.BatchNorm2d(3)
            self.dict_norm = torch.nn.ModuleDict(
                {
                    "0": torch.nn.BatchNorm2d(2),
                }
            )
            self.list_norm = torch.nn.ModuleList(
                [
                    torch.nn.BatchNorm2d(2),
                ]
            )

    return MockModel()


@e2e_pytest_unit
def test_convert_batchnorm():
    mock_model = torch.nn.SyncBatchNorm.convert_sync_batchnorm(create_model())
    assert isinstance(mock_model.norm, torch.nn.SyncBatchNorm)
    assert isinstance(mock_model.dict_norm["0"], torch.nn.SyncBatchNorm)
    assert isinstance(mock_model.list_norm[0], torch.nn.SyncBatchNorm)

    mock_model = sync_batchnorm_2_batchnorm(mock_model, 1)
    assert isinstance(mock_model.norm, torch.nn.BatchNorm1d)
    assert isinstance(mock_model.dict_norm["0"], torch.nn.BatchNorm1d)
    assert isinstance(mock_model.list_norm[0], torch.nn.BatchNorm1d)

    mock_model = torch.nn.SyncBatchNorm.convert_sync_batchnorm(create_model())
    assert isinstance(mock_model.norm, torch.nn.SyncBatchNorm)
    assert isinstance(mock_model.dict_norm["0"], torch.nn.SyncBatchNorm)
    assert isinstance(mock_model.list_norm[0], torch.nn.SyncBatchNorm)

    mock_model = sync_batchnorm_2_batchnorm(mock_model, 2)
    assert isinstance(mock_model.norm, torch.nn.BatchNorm2d)
    assert isinstance(mock_model.dict_norm["0"], torch.nn.BatchNorm2d)
    assert isinstance(mock_model.list_norm[0], torch.nn.BatchNorm2d)

    mock_model = torch.nn.SyncBatchNorm.convert_sync_batchnorm(create_model())
    assert isinstance(mock_model.norm, torch.nn.SyncBatchNorm)
    assert isinstance(mock_model.dict_norm["0"], torch.nn.SyncBatchNorm)
    assert isinstance(mock_model.list_norm[0], torch.nn.SyncBatchNorm)

    mock_model = sync_batchnorm_2_batchnorm(mock_model, 3)
    assert isinstance(mock_model.norm, torch.nn.BatchNorm3d)
    assert isinstance(mock_model.dict_norm["0"], torch.nn.BatchNorm3d)
    assert isinstance(mock_model.list_norm[0], torch.nn.BatchNorm3d)


@e2e_pytest_unit
def test_numpy2list():
    assert (0,) == numpy_2_list((0,))
    assert [0] == numpy_2_list([0])
    assert 0 == numpy_2_list(0)
    assert {0: 0} == numpy_2_list({0: 0})
    assert [0] == numpy_2_list(np.array([0]))
    assert 0 == numpy_2_list(np.array(0))
    assert {0: 0} == numpy_2_list({0: np.array(0)})



"""EfficientNet-V2 config for semi-supervised multi-label classification."""

# pylint: disable=invalid-name

_base_ = ["../../../../../recipes/stages/classification/multilabel/semisl.yaml", "../../base/models/efficientnet_v2.py"]

model = dict(
    task="classification",
    type="SemiSLMultilabelClassifier",
    backbone=dict(
        version="s_21k",
    ),
    head=dict(
        type="SemiLinearMultilabelClsHead",
        use_dynamic_loss_weighting=True,
        unlabeled_coef=0.1,
        in_channels=-1,
        aux_mlp=dict(hid_channels=0, out_channels=1024),
        normalized=True,
        scale=7.0,
        loss=dict(type="AsymmetricAngularLossWithIgnore", gamma_pos=0.0, gamma_neg=1.0, reduction="sum"),
        aux_loss=dict(
            type="BarlowTwinsLoss",
            off_diag_penality=1.0 / 128.0,
            loss_weight=1.0,
        ),
    ),
)

fp16 = dict(loss_scale=512.0)



# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from enum import Enum
from pathlib import Path

import pytest
import yaml
from omegaconf import DictConfig

from otx.api.configuration.helper.utils import (
    _search_in_config_dict_inner,
    deserialize_enum_value,
    ids_to_strings,
    input_to_config_dict,
    search_in_config_dict,
)
from otx.api.entities.id import ID
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestUtilsFunctions:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_utils_search_in_config_dict_inner(self):
        """
        <b>Description:</b>
        Check "_search_in_config_dict_inner" function

        <b>Input data:</b>
        "config_dict" dictionary, "key_to_search" string, "prior_keys" list, "results" list

        <b>Expected results:</b>
        Test passes if list returned by "_search_in_config_dict_inner" function is equal to expected

        <b>Steps</b>
        1. Check list returned by "_search_in_config_dict_inner" function with default values of optional parameters
        2. Check list returned by "_search_in_config_dict_inner" function with specified "prior_keys" parameter
        3. Check list returned by "_search_in_config_dict_inner" function with specified "results" parameter
        4. Check list returned by "_search_in_config_dict_inner" function with specified both optional parameters
        5. Check list returned by "_search_in_config_dict_inner" function with list object specified as "config_dict"
        parameter
        """
        # Checking list returned by "_search_in_config_dict_inner" with default values of optional parameters
        config_dict = {"key_1": 2, "key_2": 4, "key_3": 8}
        # Checking search of existing key
        assert _search_in_config_dict_inner(config_dict=config_dict, key_to_search="key_2") == [(4, [])]
        # Checking search of non-existing key
        assert _search_in_config_dict_inner(config_dict=config_dict, key_to_search="key_4") == []

        # Checking list returned by "_search_in_config_dict_inner" with specified "prior_keys"
        prior_keys = ["prior_key_1", "prior_key_2"]
        # Checking search of existing key
        assert _search_in_config_dict_inner(config_dict=config_dict, key_to_search="key_2", prior_keys=prior_keys) == [
            (4, prior_keys)
        ]
        # Checking search of non-existing key
        assert _search_in_config_dict_inner(config_dict=config_dict, key_to_search="key_4", prior_keys=prior_keys) == []
        # Checking list returned by "_search_in_config_dict_inner" with specified "results"
        # Checking search of existing key
        results = [(4, ["result_key", "other_result_key"])]
        expected_results = [(4, ["result_key", "other_result_key"]), (2, [])]
        assert (
            _search_in_config_dict_inner(config_dict=config_dict, key_to_search="key_1", results=results)
            == expected_results
        )
        assert results == expected_results
        # Checking search of non-existing key
        results = [(4, ["result_key", "other_result_key"])]
        expected_results = list(results)
        assert (
            _search_in_config_dict_inner(config_dict=config_dict, key_to_search="key_4", results=results)
            == expected_results
        )
        assert results == expected_results
        # Checking list returned by "_search_in_config_dict_inner" with specified "prior_keys" and "results"
        # Checking search of existing key
        results = [(4, ["result_key", "other_result_key"])]
        expected_results = [
            (4, ["result_key", "other_result_key"]),
            (8, ["prior_key_1", "prior_key_2"]),
        ]
        assert (
            _search_in_config_dict_inner(
                config_dict=config_dict,
                key_to_search="key_3",
                prior_keys=prior_keys,
                results=results,
            )
            == expected_results
        )
        assert results == expected_results
        # Checking search of non-existing key
        results = [(4, ["result_key", "other_result_key"])]
        expected_results = list(results)
        assert (
            _search_in_config_dict_inner(
                config_dict=config_dict,
                key_to_search="key_4",
                prior_keys=prior_keys,
                results=results,
            )
            == expected_results
        )
        assert results == expected_results
        # Checking list returned by "_search_in_config_dict_inner" with list specified as "config_dict"
        # Checking search of existing key
        config_list = [1, 3, 9]
        results = [(4, ["result_key", "other_result_key"])]
        expected_results = [
            (4, ["result_key", "other_result_key"]),
            (3, ["prior_key_1", "prior_key_2"]),
        ]
        assert (
            _search_in_config_dict_inner(
                config_dict=config_list,  # type: ignore
                key_to_search=1,  # type: ignore
                prior_keys=prior_keys,
                results=results,
            )
            == expected_results
        )
        assert results == expected_results
        # Checking search of non-existing key
        results = [(4, ["result_key", "other_result_key"])]
        expected_results = list(results)
        assert (
            _search_in_config_dict_inner(
                config_dict=config_list,  # type: ignore
                key_to_search=5,  # type: ignore
                prior_keys=prior_keys,
                results=results,
            )
            == expected_results
        )
        assert results == expected_results

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_utils_search_in_config_dict(self):
        """
        <b>Description:</b>
        Check "search_in_config_dict" function

        <b>Input data:</b>
        "config_dict" dictionary, "key_to_search" string

        <b>Expected results:</b>
        Test passes if list returned by "search_in_config_dict" function is equal to expected

        <b>Steps</b>
        1. Check list returned by "search_in_config_dict" function when searching key in "config_dict" dictionary
        2. Check list returned by "search_in_config_dict" function when searching key in "config_dict" list
        """
        # Checking list returned by "search_in_config_dict" when searching key in dictionary "config_dict"
        config_dict = {"key_1": 2, "key_2": 4, "key_3": 8}
        assert search_in_config_dict(config_dict, "key_1") == [(2, [])]
        # Checking search of non-existing key
        assert search_in_config_dict(config_dict, "key_4") == []
        # Checking list returned by "search_in_config_dict" when searching key in list "config_dict"
        config_dict = [1, 3, 9]
        assert search_in_config_dict(config_dict, 1) == [(3, [])]  # type: ignore
        # Checking search of non-existing key
        assert search_in_config_dict(config_dict, "key_1") == []  # type: ignore

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_utils_input_to_config_dict(self):
        """
        <b>Description:</b>
        Check "input_to_config_dict" function

        <b>Input data:</b>
        "input_config" string, DictConfig or dictionary object, "check_config_type" bool value

        <b>Expected results:</b>
        Test passes if dictionary returned by "input_to_config_dict" function is equal to expected

        <b>Steps</b>
        1. Check dictionary returned by "input_to_config_dict" function when "check_config_type" parameter is "True"
        2. Check dictionary returned by "input_to_config_dict" function when "check_config_type" parameter is "False"
        and "type" key is not specified in "input_config" parameter
        3. Check that ValueError exception is raised when type of "input_config" parameter is not equal to string,
        DictConfig or dictionary
        4. Check that ValueError exception is raised by "input_to_config_dict" function when "check_config_type"
        parameter is "True" and "type" key is not specified in "input_config" parameter
        5. Check that ValueError exception is raised by "input_to_config_dict" function when "check_config_type"
        parameter is "True" and "type" key in "input_config" parameter is equal to unexpected value
        """
        # Checking dictionary returned by "input_to_config_dict" when "check_config_type" is "True"
        path_to_config = str(Path(__file__).parent / Path(r"../dummy_config.yaml"))
        with open(path_to_config, "r", encoding="UTF-8") as file:
            expected_path_to_config_dict = yaml.safe_load(file)
        string_config = "{'str_key_1': 2, 'str_key_2': 4, 'str_key_3': 8, 'type': PARAMETER_GROUP}"
        expected_string_config_dict = {
            "str_key_1": 2,
            "str_key_2": 4,
            "str_key_3": 8,
            "type": "PARAMETER_GROUP",
        }
        dict_config = {
            "dict_key_1": 2,
            "dict_key_2": 4,
            "dict_key_3": 8,
            "type": "PARAMETER_GROUP",
        }
        expected_dict_config = {
            "dict_key_1": 2,
            "dict_key_2": 4,
            "dict_key_3": 8,
            "type": "PARAMETER_GROUP",
        }
        dict_config_instance = DictConfig(
            content={
                "DictConfig_key_1": 2,
                "DictConfig_key_2": 4,
                "DictConfig_key_3": 8,
                "type": "PARAMETER_GROUP",
            }
        )
        expected_dict_config_instance_dict = {
            "DictConfig_key_1": 2,
            "DictConfig_key_2": 4,
            "DictConfig_key_3": 8,
            "type": "PARAMETER_GROUP",
        }

        for input_config, expected_dict in [
            (path_to_config, expected_path_to_config_dict),
            (string_config, expected_string_config_dict),
            (dict_config, expected_dict_config),
            (dict_config_instance, expected_dict_config_instance_dict),
        ]:
            assert input_to_config_dict(input_config=input_config, check_config_type=True) == expected_dict
        # Checking dictionary returned by "input_to_config_dict" when "check_config_type" is "False" and "type" key
        # is not specified in "input_config"
        string_config = "{'str_key_1': 2, 'str_key_2': 4, 'str_key_3': 8}"
        expected_string_config_dict = {"str_key_1": 2, "str_key_2": 4, "str_key_3": 8}
        dict_config = {"dict_key_1": 2, "dict_key_2": 4, "dict_key_3": 8}
        expected_dict_config = {"dict_key_1": 2, "dict_key_2": 4, "dict_key_3": 8}
        dict_config_instance = DictConfig(
            content={
                "DictConfig_key_1": 2,
                "DictConfig_key_2": 4,
                "DictConfig_key_3": 8,
            }
        )
        expected_dict_config_instance_dict = {
            "DictConfig_key_1": 2,
            "DictConfig_key_2": 4,
            "DictConfig_key_3": 8,
        }

        for input_config, expected_dict in [
            (path_to_config, expected_path_to_config_dict),
            (string_config, expected_string_config_dict),
            (dict_config, expected_dict_config),
            (dict_config_instance, expected_dict_config_instance_dict),
        ]:
            assert input_to_config_dict(input_config=input_config, check_config_type=False) == expected_dict
        # Checking that ValueError exception is raised when type of "input_config" is not equal to string, DictConfig or
        # dictionary
        with pytest.raises(ValueError):
            input_to_config_dict(input_config=1)  # type: ignore
        # Checking that ValueError exception is raised by "input_to_config_dict" when "check_config_type" is "True" and
        # "type" key is not specified in "input_config"
        for none_type_input_config in [
            "{'key_1': 2, 'key_2': 4, 'key_3': 8}",
            {"key_1": 2, "key_2": 4, "key_3": 8},
            DictConfig(content={"key_1": 2, "key_2": 4, "key_3": 8}),
        ]:
            with pytest.raises(ValueError):
                input_to_config_dict(input_config=none_type_input_config)
        # Check that ValueError exception is raised by "input_to_config_dict" when "check_config_type" is "True" and
        # "type" key in "input_config" is equal to unexpected value
        for none_type_input_config in [
            "{'key_1': 2, 'key_2': 4, 'key_3': 8, 'type': unexpected_type}",
            {"key_1": 2, "key_2": 4, "key_3": 8, "type": "unexpected_type"},
            DictConfig(content={"key_1": 2, "key_2": 4, "key_3": 8, "type": "unexpected_type"}),
        ]:
            with pytest.raises(ValueError):
                input_to_config_dict(input_config=none_type_input_config)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_utils_deserialize_enum_value(self):
        """
        <b>Description:</b>
        Check "deserialize_enum_value" function

        <b>Input data:</b>
        "enum_type" Enum object, "value" Enum element or string object

        <b>Expected results:</b>
        Test passes if value returned by "deserialize_enum_value" function is equal to expected

        <b>Steps</b>
        1. Check value returned by "deserialize_enum_value" function when Enum class element is specified as "value"
        parameter
        2. Check value returned by "deserialize_enum_value" function when string is specified as "value" parameter
        3. Check that ValueError exception is raised when type of "value" parameter for "deserialize_enum_value"
        function is not equal to Enum or string
        """

        class ValidationEnum(Enum):
            FIRST = "first element"
            SECOND = "second element"

        # Checking value returned by "deserialize_enum_value" when Enum class element is specified as "value"
        assert deserialize_enum_value(value=ValidationEnum.FIRST, enum_type=ValidationEnum) == ValidationEnum.FIRST
        # Checking value returned by "deserialize_enum_value" when string is specified as "value"
        assert deserialize_enum_value(value="SECOND", enum_type=ValidationEnum) == ValidationEnum.SECOND
        with pytest.raises(KeyError):
            deserialize_enum_value(value="THIRD", enum_type=ValidationEnum)
        # Checking that ValueError exception is raised when type of "value" for "deserialize_enum_value" is not equal to
        # Enum or string
        with pytest.raises(ValueError):
            deserialize_enum_value(value=1, enum_type=ValidationEnum)  # type: ignore

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_utils_ids_to_strings(self):
        """
        <b>Description:</b>
        Check "ids_to_strings" function

        <b>Input data:</b>
        "config_dict" dictionary

        <b>Expected results:</b>
        Test passes if dictionary returned by "ids_to_strings" function is equal to expected
        """
        config_dict = {
            "id_key_1": ID("1"),
            "not_id_key_1": 2,
            "not_id_key_2": 3,
            "id_key_2": ID("4"),
        }
        expected_dict = {
            "id_key_1": "1",
            "not_id_key_1": 2,
            "not_id_key_2": 3,
            "id_key_2": "4",
        }
        assert ids_to_strings(config_dict) == expected_dict
        assert config_dict == expected_dict



"""Cross entropy loss for ignored mode in class-incremental learning."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from typing import Optional

import torch
import torch.nn.functional as F
from mmseg.models.builder import LOSSES
from mmseg.models.losses import CrossEntropyLoss
from mmseg.models.losses.utils import weight_reduce_loss


@LOSSES.register_module()
class CrossEntropyLossWithIgnore(CrossEntropyLoss):
    """CrossEntropyLossWithIgnore with Ignore Mode Support for Class Incremental Learning.

    When new classes are added through continual training cycles, images from previous cycles
    may become partially annotated if they are not revisited.
    To prevent the model from predicting these new classes for such images,
    CrossEntropyLossWithIgnore can be used to ignore the unseen classes.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._loss_name = "loss_ce_ignore"

    def forward(
        self,
        cls_score: Optional[torch.Tensor],
        label: Optional[torch.Tensor],
        weight: Optional[torch.Tensor] = None,
        avg_factor: Optional[int] = None,
        reduction_override: Optional[str] = "mean",
        ignore_index: int = 255,
        valid_label_mask: Optional[torch.Tensor] = None,
        **kwargs
    ):
        """Forward.

        Args:
            cls_score (torch.Tensor, optional): The prediction with shape (N, 1).
            label (torch.Tensor, optional): The learning label of the prediction.
            weight (torch.Tensor, optional): Sample-wise loss weight.
                Default: None.
            class_weight (list[float], optional): The weight for each class.
                Default: None.
            avg_factor (int, optional): Average factor that is used to average
                the loss. Default: None.
            reduction_override (str, optional): The method used to reduce the loss.
                Options are 'none', 'mean' and 'sum'. Default: 'mean'.
            ignore_index (int): Specifies a target value that is ignored and
                does not contribute to the input gradients. When
                ``avg_non_ignore `` is ``True``, and the ``reduction`` is
                ``''mean''``, the loss is averaged over non-ignored targets.
                Defaults: 255.
            valid_label_mask (torch.Tensor, optional): The valid labels with
                shape (N, num_classes).
                If the value in the valid_label_mask is 0, mask label of the
                the mask label of the class corresponding to its index will be
                ignored like ignore_index.
            **kwargs (Any): Additional keyword arguments.
        """
        if valid_label_mask is None:
            losses = super().forward(cls_score, label, weight, avg_factor, reduction_override, ignore_index, **kwargs)
            return losses
        else:
            assert reduction_override in (None, "none", "mean", "sum")
            reduction = reduction_override if reduction_override else self.reduction
            batch_size = label.shape[0]
            for i in range(batch_size):
                invalid_labels = (valid_label_mask[i] == 0).nonzero(as_tuple=False)

                for inv_l in invalid_labels:
                    label[i] = torch.where(label[i] == inv_l.item(), ignore_index, label[i])

            losses = F.cross_entropy(cls_score, label, reduction="none", ignore_index=ignore_index)

            if weight is not None:
                weight = weight.float()
            losses = weight_reduce_loss(losses, weight=weight, reduction=reduction, avg_factor=avg_factor)

            return losses

    @property
    def loss_name(self):
        """Loss Name.

        This function must be implemented and will return the name of this
        loss function. This name will be used to combine different loss items
        by simple sum operation. In addition, if you want this loss item to be
        included into the backward graph, `loss_` must be the prefix of the
        name.

        Returns:
            str: The name of this loss item.
        """
        return self._loss_name



"""Model configuration of MoViNet model for Action Classification Task."""

# Copyright (C) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

# pylint: disable=invalid-name

_base_ = ["../base/supervised.py"]

num_classes = 400
model = dict(
    type="MoViNetRecognizer",
    backbone=dict(type="OTXMoViNet"),
    cls_head=dict(
        type="MoViNetHead",
        in_channels=480,
        hidden_dim=2048,
        num_classes=num_classes,
        loss_cls=dict(type="CrossEntropyLoss", loss_weight=1.0),
    ),
    # model training and testing settings
    train_cfg=None,
    test_cfg=dict(average_clips="prob"),
)

resume_from = None
load_from = "https://github.com/Atze00/MoViNet-pytorch/blob/main/weights/modelA0_statedict_v3?raw=true"



import json
from os import path as osp

import pytest

from otx.hpo.hpo_base import Trial
from tests.test_suite.e2e_test_system import e2e_pytest_component


@pytest.fixture
def good_trial_args():
    return {"trial_id": "name", "configuration": {"hp1": 1, "hp2": 1.2}, "train_environment": {"subset_ratio": 0.5}}


@pytest.fixture
def trial(good_trial_args):
    return Trial(**good_trial_args)


def register_scores_to_trial(trial, scores=[val for val in range(100)]):
    if len(trial.score) != 0:
        base_resource = max(trial.score.keys())
    else:
        base_resource = 0
    for idx, score in enumerate(scores):
        trial.register_score(score, base_resource + idx + 1)


class TestTrial:
    @e2e_pytest_component
    def test_init(self, good_trial_args):
        Trial(**good_trial_args)

    @e2e_pytest_component
    def test_set_iteration(self, trial):
        trial.iteration = 10
        config = trial.get_train_configuration()
        assert config["configuration"]["iterations"] == 10

    @e2e_pytest_component
    @pytest.mark.parametrize("iter_val", [-10, 0])
    def test_set_negative_iteration(self, trial, iter_val):
        with pytest.raises(ValueError):
            trial.iteration = iter_val

    @e2e_pytest_component
    def test_get_train_configuration(self, good_trial_args):
        trial = Trial(**good_trial_args)
        train_config = trial.get_train_configuration()
        assert train_config["configuration"] == good_trial_args["configuration"]
        assert train_config["train_environment"] == good_trial_args["train_environment"]

    @e2e_pytest_component
    def test_get_train_configuration_without_train_env(self, good_trial_args):
        del good_trial_args["train_environment"]
        trial = Trial(**good_trial_args)
        train_config = trial.get_train_configuration()
        assert train_config["train_environment"] is None

    @e2e_pytest_component
    @pytest.mark.parametrize("score", [-10, 12.5])
    def test_register_score(self, trial, score):
        for resource in [1, 4.3, 10]:
            trial.register_score(score, resource)

    @e2e_pytest_component
    @pytest.mark.parametrize("resource", [-10, 0])
    def test_register_score_not_postive_resource(self, trial, resource):
        score = 10
        with pytest.raises(ValueError):
            trial.register_score(score, resource)

    @e2e_pytest_component
    @pytest.mark.parametrize("mode", ["min", "max"])
    @pytest.mark.parametrize("resource_limit", [None, 10, 20])
    def test_get_best_score(self, trial, mode, resource_limit):
        scores = [val for val in range(100)]
        register_scores_to_trial(trial, scores)

        if resource_limit is not None:
            scores = {i + 1: score for i, score in enumerate(scores)}
            scores = [val for key, val in scores.items() if key <= resource_limit]

        if mode == "min":
            expected_score = min(scores)
        else:
            expected_score = max(scores)

        assert expected_score == trial.get_best_score(mode, resource_limit)

    @e2e_pytest_component
    def test_get_best_score_empty_score(self, trial):
        assert trial.get_best_score() is None

    @e2e_pytest_component
    def test_get_best_score_no_trial_to_meet_condition(self, trial):
        scores = [val for val in range(100)]
        register_scores_to_trial(trial, scores)
        assert trial.get_best_score(resource_limit=0.5) is None

    @e2e_pytest_component
    def test_get_best_score_with_empty_scores(self, trial):
        assert trial.get_best_score() is None

    @e2e_pytest_component
    def test_get_best_score_with_wrong_mode_value(self, trial):
        register_scores_to_trial(trial)
        with pytest.raises(ValueError):
            trial.get_best_score(mode="wrong")

    @e2e_pytest_component
    @pytest.mark.parametrize("resource", [12, 42.12])
    def test_get_progress(self, trial, resource):
        trial.register_score(100, resource)
        assert trial.get_progress() == resource

    @e2e_pytest_component
    def test_get_progress_not_trained_at_all(self, trial):
        assert trial.get_progress() == 0

    @e2e_pytest_component
    def test_save_results(self, trial, tmp_path):
        register_scores_to_trial(trial)
        save_path = osp.join(tmp_path, "test")
        trial.save_results(save_path)

        with open(save_path, "r") as f:
            result = json.load(f)

        assert result["id"] == "name"
        assert result["configuration"]["hp1"] == 1
        assert result["configuration"]["hp2"] == 1.2
        assert result["train_environment"]["subset_ratio"] == 0.5
        for key, val in result["score"].items():
            assert int(key) - 1 == val

    @e2e_pytest_component
    def test_finalize(self, trial):
        trial.iteration = 10
        trial.register_score(10, 5)
        trial.finalize()
        assert trial.iteration == trial.get_progress()

    @e2e_pytest_component
    def test_finalize_without_registered_score(self, trial):
        trial.iteration = 10
        with pytest.raises(RuntimeError):
            trial.finalize()

    @e2e_pytest_component
    def test_is_not_done(self, trial):
        trial.iteration = 10
        assert not trial.is_done()

    @e2e_pytest_component
    def test_is_done(self, trial):
        trial.iteration = 10
        trial.register_score(10, 10)
        assert trial.is_done()

    @e2e_pytest_component
    def test_is_done_iteration_not_set_yet(self, trial):
        with pytest.raises(ValueError):
            trial.is_done()



"""Logger hooks."""
from collections import defaultdict
from typing import Any, Dict, Optional

from mmcv.runner import BaseRunner
from mmcv.runner.dist_utils import master_only
from mmcv.runner.hooks import HOOKS, Hook, LoggerHook

from otx.algorithms.common.utils.logger import get_logger

logger = get_logger()


@HOOKS.register_module()
class OTXLoggerHook(LoggerHook):
    """OTXLoggerHook for Logging."""

    class Curve:
        """Curve with x (epochs) & y (scores)."""

        def __init__(self):
            self.x = []
            self.y = []

        def __repr__(self):
            """Repr function."""
            points = []
            for x, y in zip(self.x, self.y):
                points.append(f"({x},{y})")
            return "curve[" + ",".join(points) + "]"

    def __init__(
        self,
        curves: Optional[Dict[Any, Curve]] = None,
        interval: int = 10,
        ignore_last: bool = True,
        reset_flag: bool = True,
        by_epoch: bool = True,
    ):
        super().__init__(interval, ignore_last, reset_flag, by_epoch)
        self.curves = curves if curves is not None else defaultdict(self.Curve)

    @master_only
    def log(self, runner: BaseRunner):
        """Log function for OTXLoggerHook."""
        tags = self.get_loggable_tags(runner, allow_text=False, tags_to_skip=())
        if runner.max_epochs is not None:
            normalized_iter = self.get_iter(runner) / runner.max_iters * runner.max_epochs
        else:
            normalized_iter = self.get_iter(runner)
        for tag, value in tags.items():
            curve = self.curves[tag]
            # Remove duplicates.
            if len(curve.x) > 0 and curve.x[-1] == normalized_iter:
                curve.x.pop()
                curve.y.pop()
            curve.x.append(normalized_iter)
            curve.y.append(value)

    def after_train_epoch(self, runner: BaseRunner):
        """Called after_train_epoch in OTXLoggerHook."""
        # Iteration counter is increased right after the last iteration in the epoch,
        # temporarily decrease it back.
        runner._iter -= 1
        super().after_train_epoch(runner)
        runner._iter += 1


@HOOKS.register_module()
class LoggerReplaceHook(Hook):
    """replace logger in the runner to the MPA logger.

    DO NOT INCLUDE this hook to the recipe directly.
    mpa will add this hook to all recipe internally.
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def before_run(self, runner):
        """Replace logger."""
        runner.logger = logger
        logger.info("logger in the runner is replaced to the MPA logger")



# Copyright (C) 2021-2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import os
import tempfile

import pytest
import numpy as np
import torch
from nncf.torch.nncf_network import NNCFNetwork

from otx.algorithms.common.adapters.nncf.compression import NNCFMetaState
from otx.algorithms.detection.adapters.mmdet.nncf.builder import build_nncf_detector
from tests.test_suite.e2e_test_system import e2e_pytest_unit
from tests.unit.algorithms.common.adapters.mmcv.nncf.test_helpers import (
    create_config,
    create_dataset,
    create_model,
)


@pytest.fixture(autouse=True)
def prepare_dataset():
    create_dataset(lib="mmdet")


@pytest.fixture(scope="module")
def temp_dir():
    with tempfile.TemporaryDirectory() as tempdir:
        yield tempdir


@pytest.fixture
def nncf_model_path(temp_dir):
    return os.path.join(temp_dir, "nncf_model.bin")


@pytest.fixture(scope="module")
def state_to_build():
    model = create_model(lib="mmdet")
    return model.state_dict()


@pytest.fixture
def mock_config(temp_dir, state_to_build):
    model_path = os.path.join(temp_dir, "model.bin")
    mock_config = create_config(lib="mmdet")
    torch.save(state_to_build, model_path)
    mock_config.load_from = model_path
    return mock_config


@e2e_pytest_unit
def test_build_nncf_detector(mock_config):
    with tempfile.TemporaryDirectory() as tempdir:
        mock_config.nncf_config.log_dir = tempdir
        _, model = build_nncf_detector(mock_config)

    assert isinstance(model, NNCFNetwork)
    assert len([hook for hook in mock_config.custom_hooks if hook.type == "CompressionHook"]) == 1


@e2e_pytest_unit
def test_build_nncf_detector_not_compress_postprocessing(mock_config, state_to_build, nncf_model_path):
    with tempfile.TemporaryDirectory() as tempdir:
        mock_config.nncf_config.log_dir = tempdir
        mock_config.nncf_compress_postprocessing = False
        ctrl, model = build_nncf_detector(mock_config)

    assert isinstance(model, NNCFNetwork)
    assert len([hook for hook in mock_config.custom_hooks if hook.type == "CompressionHook"]) == 1

    # save a model for next test
    torch.save(
        {
            "meta": {
                "nncf_enable_compression": True,
                "nncf_meta": NNCFMetaState(
                    data_to_build=np.zeros((50, 50, 3)),
                    compression_ctrl=ctrl.get_compression_state(),
                    state_to_build=state_to_build,
                ),
            },
            "state_dict": model.state_dict(),
        },
        nncf_model_path,
    )


@e2e_pytest_unit
def test_build_nncf_detector_with_nncf_ckpt(mock_config, nncf_model_path):
    with tempfile.TemporaryDirectory() as tempdir:
        mock_config.nncf_config.log_dir = tempdir
        _, model = build_nncf_detector(mock_config, nncf_model_path)
    assert isinstance(model, NNCFNetwork)
    assert len([hook for hook in mock_config.custom_hooks if hook.type == "CompressionHook"]) == 1



import json
import os

from tests.test_suite.run_test_command import (
    get_template_dir,
    check_run,
)


def regression_eval_testing(
    template,
    root,
    otx_dir,
    args,
    criteria,
    result_dict,
    threshold=0.10,
):
    regression_result = {
        "passed": True,
        "log": "",
    }

    template_work_dir = get_template_dir(template, root)

    command_line = [
        "otx",
        "eval",
        template.model_template_path,
        "--test-data-roots",
        f'{os.path.join(otx_dir, args["--test-data-roots"])}',
        "--load-weights",
        f"{template_work_dir}/trained_{template.model_template_id}/models/weights.pth",
        "--output",
        f"{template_work_dir}/trained_{template.model_template_id}",
    ]
    command_line.extend(["--workspace", f"{template_work_dir}"])
    command_line.extend(args.get("eval_params", []))
    check_run(command_line)

    performance_json_path = f"{template_work_dir}/trained_{template.model_template_id}/performance.json"
    assert os.path.exists(performance_json_path)

    with open(performance_json_path) as read_file:
        trained_performance = json.load(read_file)

    model_criteria = criteria[template.name]
    modified_criteria = model_criteria - (model_criteria * threshold)
    for k in trained_performance.keys():
        result_dict[k] = round(trained_performance[k], 3)
        if trained_performance[k] < modified_criteria:
            regression_result["passed"] = False
            regression_result["log"] = f"Performance: ({trained_performance[k]}) < Criteria: ({modified_criteria})."

    result_dict["Model size (MB)"] = round(
        os.path.getsize(f"{template_work_dir}/trained_{template.model_template_id}/models/weights.pth") / 1e6, 2
    )

    return regression_result


def regression_openvino_testing(
    template,
    root,
    otx_dir,
    args,
    threshold=0.0,
    criteria=None,
    reg_threshold=0.10,
    result_dict=None,
    half_precision=False,
):
    regression_result = {
        "passed": True,
        "log": "",
    }

    template_work_dir = get_template_dir(template, root)
    weights_path = f"{template_work_dir}/exported_{template.model_template_id}/openvino.xml"
    output_path = f"{template_work_dir}/exported_{template.model_template_id}"
    perf_path = f"{template_work_dir}/exported_{template.model_template_id}/performance.json"

    if half_precision:
        weights_path = f"{template_work_dir}/exported_{template.model_template_id}_fp16/openvino.xml"
        output_path = f"{template_work_dir}/exported_{template.model_template_id}_fp16"
        perf_path = f"{template_work_dir}/exported_{template.model_template_id}_fp16/performance.json"

    command_line = [
        "otx",
        "eval",
        template.model_template_path,
        "--test-data-roots",
        f'{os.path.join(otx_dir, args["--test-data-roots"])}',
        "--load-weights",
        weights_path,
        "--output",
        output_path,
    ]
    command_line.extend(["--workspace", f"{template_work_dir}"])
    check_run(command_line)
    assert os.path.exists(perf_path)
    with open(f"{template_work_dir}/trained_{template.model_template_id}/performance.json") as read_file:
        trained_performance = json.load(read_file)
    with open(perf_path) as read_file:
        exported_performance = json.load(read_file)

    if isinstance(criteria, dict) and template.name in criteria.keys():
        model_criteria = criteria[template.name]
        modified_criteria = model_criteria - (model_criteria * reg_threshold)

    for k in trained_performance.keys():
        if isinstance(criteria, dict) and template.name in criteria.keys():
            result_dict[k] = round(exported_performance[k], 3)
            if exported_performance[k] < modified_criteria:
                regression_result["passed"] = False
                regression_result[
                    "log"
                ] = f"Export performance: ({exported_performance[k]}) < Criteria: ({modified_criteria})."

        if (
            exported_performance[k] < trained_performance[k]
            and abs(trained_performance[k] - exported_performance[k]) / (trained_performance[k] + 1e-10) > threshold
        ):
            regression_result["passed"] = False
            regression_result["log"] = f"{trained_performance[k]=}, {exported_performance[k]=}"

    return regression_result


def regression_deployment_testing(
    template, root, otx_dir, args, threshold=0.0, criteria=None, reg_threshold=0.10, result_dict=None
):
    regression_result = {
        "passed": True,
        "log": "",
    }

    template_work_dir = get_template_dir(template, root)
    command_line = [
        "otx",
        "eval",
        template.model_template_path,
        "--test-data-roots",
        f'{os.path.join(otx_dir, args["--test-data-roots"])}',
        "--load-weights",
        f"{template_work_dir}/deployed_{template.model_template_id}/openvino.zip",
        "--output",
        f"{template_work_dir}/deployed_{template.model_template_id}",
    ]
    command_line.extend(["--workspace", f"{template_work_dir}"])
    check_run(command_line)
    assert os.path.exists(f"{template_work_dir}/deployed_{template.model_template_id}/performance.json")
    with open(f"{template_work_dir}/exported_{template.model_template_id}/performance.json") as read_file:
        exported_performance = json.load(read_file)
    with open(f"{template_work_dir}/deployed_{template.model_template_id}/performance.json") as read_file:
        deployed_performance = json.load(read_file)

    if isinstance(criteria, dict) and template.name in criteria.keys():
        model_criteria = criteria[template.name]
        modified_criteria = model_criteria - (model_criteria * reg_threshold)

    for k in exported_performance.keys():
        if isinstance(criteria, dict) and template.name in criteria.keys():
            result_dict[k] = round(deployed_performance[k], 3)
            if exported_performance[k] < modified_criteria:
                regression_result["passed"] = False
                regression_result[
                    "log"
                ] = f"Deploy performance: ({deployed_performance[k]}) < Criteria: ({modified_criteria})."
        if (
            deployed_performance[k] < exported_performance[k]
            and abs(exported_performance[k] - deployed_performance[k]) / (exported_performance[k] + 1e-10) > threshold
        ):
            regression_result["passed"] = False
            regression_result["log"] = f"{exported_performance[k]=}, {deployed_performance[k]=}"

    return regression_result


def regression_nncf_eval_testing(
    template, root, otx_dir, args, threshold=0.01, criteria=None, reg_threshold=0.10, result_dict=None
):
    regression_result = {
        "passed": True,
        "log": "",
    }

    template_work_dir = get_template_dir(template, root)
    command_line = [
        "otx",
        "eval",
        template.model_template_path,
        "--test-data-roots",
        f'{os.path.join(otx_dir, args["--test-data-roots"])}',
        "--load-weights",
        f"{template_work_dir}/nncf_{template.model_template_id}/weights.pth",
        "--output",
        f"{template_work_dir}/nncf_{template.model_template_id}",
    ]
    command_line.extend(["--workspace", f"{template_work_dir}"])
    check_run(command_line)
    assert os.path.exists(f"{template_work_dir}/nncf_{template.model_template_id}/performance.json")
    with open(f"{template_work_dir}/nncf_{template.model_template_id}/nncf_performance.json") as read_file:
        trained_performance = json.load(read_file)
    with open(f"{template_work_dir}/nncf_{template.model_template_id}/performance.json") as read_file:
        evaluated_performance = json.load(read_file)

    if isinstance(criteria, dict) and template.name in criteria.keys():
        model_criteria = criteria[template.name]
        modified_criteria = model_criteria - (model_criteria * reg_threshold)

    for k in trained_performance.keys():
        if isinstance(criteria, dict) and template.name in criteria.keys():
            result_dict[k] = round(evaluated_performance[k], 3)
            if evaluated_performance[k] < modified_criteria:
                regression_result["passed"] = False
                regression_result[
                    "log"
                ] = f"NNCF performance: ({evaluated_performance[k]}) < Criteria: ({modified_criteria})."
        if (
            evaluated_performance[k] < trained_performance[k]
            and abs(trained_performance[k] - evaluated_performance[k]) / (trained_performance[k] + 1e-10) > threshold
        ):
            regression_result["passed"] = False
            regression_result["log"] = f"{trained_performance[k]=}, {evaluated_performance[k]=}"

    return regression_result


def regression_pot_eval_testing(template, root, otx_dir, args, criteria=None, reg_threshold=0.10, result_dict=None):
    regression_result = {
        "passed": True,
        "log": "",
    }

    template_work_dir = get_template_dir(template, root)
    command_line = [
        "otx",
        "eval",
        template.model_template_path,
        "--test-data-roots",
        f'{os.path.join(otx_dir, args["--test-data-roots"])}',
        "--load-weights",
        f"{template_work_dir}/pot_{template.model_template_id}/openvino.xml",
        "--output",
        f"{template_work_dir}/pot_{template.model_template_id}",
    ]
    command_line.extend(["--workspace", f"{template_work_dir}"])
    check_run(command_line)
    assert os.path.exists(f"{template_work_dir}/pot_{template.model_template_id}/performance.json")

    with open(f"{template_work_dir}/pot_{template.model_template_id}/performance.json") as read_file:
        pot_performance = json.load(read_file)

    if isinstance(criteria, dict) and template.name in criteria.keys():
        model_criteria = criteria[template.name]
        modified_criteria = model_criteria - (model_criteria * reg_threshold)

    for k in pot_performance.keys():
        if isinstance(criteria, dict) and template.name in criteria.keys():
            result_dict[k] = round(pot_performance[k], 3)
            if pot_performance[k] < modified_criteria:
                regression_result["passed"] = False
                regression_result["log"] = f"POT performance: ({pot_performance[k]}) < Criteria: ({modified_criteria})."

    return regression_result


def regression_train_time_testing(train_time_criteria, e2e_train_time, template, threshold=0.10):
    """Measure train+val time and comapre with test criteria.

    Test criteria was set by previous measurement.
    """
    regression_result = {
        "passed": True,
        "log": "",
    }

    e2e_train_time_criteria = train_time_criteria[template.name]
    modified_train_criteria = e2e_train_time_criteria - (e2e_train_time_criteria * threshold)

    if e2e_train_time < modified_train_criteria:
        regression_result["passed"] = False
        regression_result["log"] = f"Train time: ({e2e_train_time}) < Criteria: ({modified_train_criteria})."

    return regression_result


def regression_eval_time_testing(eval_time_criteria, e2e_eval_time, template, threshold=0.10):
    """Measure evaluation time and comapre with test criteria.

    Test criteria was set by previous measurement.
    """
    regression_result = {
        "passed": True,
        "log": "",
    }

    e2e_eval_time_criteria = eval_time_criteria[template.name]
    modified_eval_criteria = e2e_eval_time_criteria - (e2e_eval_time_criteria * threshold)

    if e2e_eval_time < modified_eval_criteria:
        regression_result["passed"] = False
        regression_result["log"] = f"Eval time: ({e2e_eval_time}) < criteria: ({modified_eval_criteria})."

    return regression_result



"""Tests for Anomaly Segmentation with OTX CLI."""
# Copyright (C) 2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#
import json
import os
import random
from pathlib import Path
from timeit import default_timer as timer

import pytest

from otx.cli.registry import Registry
from tests.regression.regression_test_helpers import (
    ANOMALY_DATASET_CATEGORIES,
    TIME_LOG,
    get_result_dict,
    get_template_performance,
    load_regression_configuration,
)
from tests.test_suite.e2e_test_system import e2e_pytest_component
from tests.test_suite.run_test_command import (
    nncf_optimize_testing,
    otx_deploy_openvino_testing,
    otx_export_testing,
    otx_train_testing,
    pot_optimize_testing,
)

from tests.regression.regression_command import (
    regression_eval_testing,
    regression_openvino_testing,
    regression_deployment_testing,
    regression_nncf_eval_testing,
    regression_pot_eval_testing,
    regression_train_time_testing,
    regression_eval_time_testing,
)

# Configurations for regression test.
TASK_TYPE = "anomaly_segmentation"
SAMPLED_ANOMALY_DATASET_CATEGORIES = random.sample(ANOMALY_DATASET_CATEGORIES, 15)

otx_dir = os.getcwd()
templates = Registry("otx/algorithms/anomaly").filter(task_type=TASK_TYPE.upper()).templates
templates_ids = [template.model_template_id for template in templates]

result_dict = get_result_dict(TASK_TYPE)
result_dir = f"/tmp/regression_test_results/{TASK_TYPE}"
Path(result_dir).mkdir(parents=True, exist_ok=True)

anomaly_segmentation_regression_config = load_regression_configuration(otx_dir, TASK_TYPE)
anomaly_segmentation_data_args = anomaly_segmentation_regression_config["data_path"]


class TestRegressionAnomalySegmentation:
    def setup_method(self):
        self.performance = {}

    def teardown_method(self):
        with open(f"{result_dir}/result.json", "w") as result_file:
            json.dump(result_dict, result_file, indent=4)

    def _apply_category(self, data_dict, category):
        return_dict = {}
        for k, v in data_dict.items():
            if "train" in k:
                return_dict[k] = f"{v}/{category}/train"
            if "val" in k or "test" in k:
                return_dict[k] = f"{v}/{category}/test"
            else:
                return_dict[k] = v
        return return_dict

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    @pytest.mark.parametrize("category", SAMPLED_ANOMALY_DATASET_CATEGORIES)
    def test_otx_train(self, template, tmp_dir_path, category):
        self.performance[template.name] = {}
        category_data_args = self._apply_category(anomaly_segmentation_data_args, category)

        tmp_dir_path = tmp_dir_path / TASK_TYPE
        train_start_time = timer()
        otx_train_testing(template, tmp_dir_path, otx_dir, category_data_args)
        train_elapsed_time = timer() - train_start_time

        infer_start_time = timer()
        test_result = regression_eval_testing(
            template,
            tmp_dir_path,
            otx_dir,
            category_data_args,
            anomaly_segmentation_regression_config["regression_criteria"]["train"][category],
            self.performance[template.name],
        )
        infer_elapsed_time = timer() - infer_start_time

        self.performance[template.name][TIME_LOG["train_time"]] = round(train_elapsed_time, 3)
        self.performance[template.name][TIME_LOG["infer_time"]] = round(infer_elapsed_time, 3)
        result_dict[TASK_TYPE]["train"][category].append(self.performance)

        assert test_result["passed"] is True, test_result["log"]

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    @pytest.mark.parametrize("category", SAMPLED_ANOMALY_DATASET_CATEGORIES)
    def test_otx_train_kpi_test(self, template, category):
        """KPI tests: measure the train+val time and evaluation time and compare with criteria."""
        results = result_dict[TASK_TYPE]["train"][category]
        performance = get_template_performance(results, template)

        # Compare train+val time with the KPI criteria.
        kpi_train_result = regression_train_time_testing(
            train_time_criteria=anomaly_segmentation_regression_config["kpi_e2e_train_time_criteria"]["train"][
                category
            ],
            e2e_train_time=performance[template.name][TIME_LOG["train_time"]],
            template=template,
        )

        # Compare evaluation time with the KPI criteria.
        kpi_eval_result = regression_eval_time_testing(
            eval_time_criteria=anomaly_segmentation_regression_config["kpi_e2e_eval_time_criteria"]["train"][category],
            e2e_eval_time=performance[template.name][TIME_LOG["infer_time"]],
            template=template,
        )

        assert kpi_train_result["passed"] is True, kpi_train_result["log"]
        assert kpi_eval_result["passed"] is True, kpi_eval_result["log"]

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    @pytest.mark.parametrize("category", SAMPLED_ANOMALY_DATASET_CATEGORIES)
    def test_otx_export_eval_openvino(self, template, tmp_dir_path, category):
        self.performance[template.name] = {}
        category_data_args = self._apply_category(anomaly_segmentation_data_args, category)

        tmp_dir_path = tmp_dir_path / TASK_TYPE
        export_start_time = timer()
        otx_export_testing(template, tmp_dir_path)
        export_elapsed_time = timer() - export_start_time

        export_eval_start_time = timer()
        test_result = regression_openvino_testing(
            template,
            tmp_dir_path,
            otx_dir,
            category_data_args,
            threshold=0.05,
            criteria=anomaly_segmentation_regression_config["regression_criteria"]["export"][category],
            reg_threshold=0.10,
            result_dict=self.performance[template.name],
        )
        export_eval_elapsed_time = timer() - export_eval_start_time

        self.performance[template.name][TIME_LOG["export_time"]] = round(export_elapsed_time, 3)
        self.performance[template.name][TIME_LOG["export_eval_time"]] = round(export_eval_elapsed_time, 3)
        result_dict[TASK_TYPE]["export"][category].append(self.performance)

        assert test_result["passed"] is True, test_result["log"]

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    @pytest.mark.parametrize("category", SAMPLED_ANOMALY_DATASET_CATEGORIES)
    def test_otx_deploy_eval_deployment(self, template, tmp_dir_path, category):
        self.performance[template.name] = {}
        category_data_args = self._apply_category(anomaly_segmentation_data_args, category)

        tmp_dir_path = tmp_dir_path / TASK_TYPE
        deploy_start_time = timer()
        otx_deploy_openvino_testing(template, tmp_dir_path, otx_dir, category_data_args)
        deploy_elapsed_time = timer() - deploy_start_time

        deploy_eval_start_time = timer()
        test_result = regression_deployment_testing(
            template,
            tmp_dir_path,
            otx_dir,
            category_data_args,
            threshold=0.0,
            criteria=anomaly_segmentation_regression_config["regression_criteria"]["deploy"][category],
            reg_threshold=0.10,
            result_dict=self.performance[template.name],
        )
        deploy_eval_elapsed_time = timer() - deploy_eval_start_time

        self.performance[template.name][TIME_LOG["deploy_time"]] = round(deploy_elapsed_time, 3)
        self.performance[template.name][TIME_LOG["deploy_eval_time"]] = round(deploy_eval_elapsed_time, 3)
        result_dict[TASK_TYPE]["deploy"][category].append(self.performance)

        assert test_result["passed"] is True, test_result["log"]

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    @pytest.mark.parametrize("category", SAMPLED_ANOMALY_DATASET_CATEGORIES)
    def test_nncf_optimize_eval(self, template, tmp_dir_path, category):
        self.performance[template.name] = {}
        category_data_args = self._apply_category(anomaly_segmentation_data_args, category)

        tmp_dir_path = tmp_dir_path / TASK_TYPE
        if template.entrypoints.nncf is None:
            pytest.skip("nncf entrypoint is none")

        nncf_start_time = timer()
        nncf_optimize_testing(template, tmp_dir_path, otx_dir, category_data_args)
        nncf_elapsed_time = timer() - nncf_start_time

        nncf_eval_start_time = timer()
        test_result = regression_nncf_eval_testing(
            template,
            tmp_dir_path,
            otx_dir,
            category_data_args,
            threshold=0.01,
            criteria=anomaly_segmentation_regression_config["regression_criteria"]["nncf"][category],
            reg_threshold=0.10,
            result_dict=self.performance[template.name],
        )
        nncf_eval_elapsed_time = timer() - nncf_eval_start_time

        self.performance[template.name][TIME_LOG["nncf_time"]] = round(nncf_elapsed_time, 3)
        self.performance[template.name][TIME_LOG["nncf_eval_time"]] = round(nncf_eval_elapsed_time, 3)
        result_dict[TASK_TYPE]["nncf"][category].append(self.performance)

        assert test_result["passed"] is True, test_result["log"]

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    @pytest.mark.parametrize("category", SAMPLED_ANOMALY_DATASET_CATEGORIES)
    def test_pot_optimize_eval(self, template, tmp_dir_path, category):
        self.performance[template.name] = {}
        category_data_args = self._apply_category(anomaly_segmentation_data_args, category)

        tmp_dir_path = tmp_dir_path / TASK_TYPE
        pot_start_time = timer()
        pot_optimize_testing(template, tmp_dir_path, otx_dir, category_data_args)
        pot_elapsed_time = timer() - pot_start_time

        pot_eval_start_time = timer()
        test_result = regression_pot_eval_testing(
            template,
            tmp_dir_path,
            otx_dir,
            category_data_args,
            criteria=anomaly_segmentation_regression_config["regression_criteria"]["pot"][category],
            reg_threshold=0.10,
            result_dict=self.performance[template.name],
        )
        pot_eval_elapsed_time = timer() - pot_eval_start_time

        self.performance[template.name][TIME_LOG["pot_time"]] = round(pot_elapsed_time, 3)
        self.performance[template.name][TIME_LOG["pot_eval_time"]] = round(pot_eval_elapsed_time, 3)
        result_dict[TASK_TYPE]["pot"][category].append(self.performance)

        assert test_result["passed"] is True, test_result["log"]



"""Base Action detection mmdeply cfg."""

_base_ = ["./base_detection_static.py"]

ir_config = dict(
    dynamic_axes=dict(
        input=dict({0: "batch", 1: "channel", 2: "clip_len", 3: "height", 4: "width"}),
        dets=dict({0: "batch", 1: "num_dets"}),
        labels=dict({0: "batch", 1: "num_dets"}),
    ),
)



# Copyright (C) 2021-2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import pytest

from otx.core.ov.registry import Registry
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class TestRegistry:
    @e2e_pytest_unit
    def test_register(self):
        registry = Registry("dummy")

        @registry.register()
        def dummy1():
            pass

        assert "dummy1" in registry.registry_dict.keys()
        assert dummy1 in registry
        assert dummy1 == registry.get("dummy1")

        @registry.register("dummy_name")
        def dummy2():
            pass

        assert "dummy_name" in registry.registry_dict.keys()
        assert dummy2 in registry
        assert dummy2 == registry.get("dummy_name")

        with pytest.raises(KeyError):
            registry.get("error")



"""Unit Test for otx.algorithms.action.adapters.mmaction.utils.config_utils."""

# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from typing import Any

import torch
from mmaction.models import Recognizer3D
from mmcv.runner import BaseModule
from mmcv.utils import Config
from torch import nn

from otx.algorithms.action.adapters.mmaction.models.detectors.fast_rcnn import (
    AVAFastRCNN,
)
from otx.algorithms.action.adapters.mmaction.utils.export_utils import (
    Exporter,
    _convert_sync_batch_to_normal_batch,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class MockRecognizer3D(Recognizer3D, BaseModule):
    """Mock class for Recognizer3D."""

    def __init__(self) -> None:
        super(BaseModule, self).__init__()

    def forward(self, inputs: Any) -> str:
        return "Forward function is replaced!"

    def load_state_dict(self, weights) -> Recognizer3D:
        pass


class MockAVAFastRCNN(AVAFastRCNN):
    """Mock class for AVAFastRCNN."""

    def __init__(self) -> None:
        super(BaseModule, self).__init__()
        self.deploy_cfg = None

    def patch_for_export(self) -> None:
        pass

    def forward_infer(self, inputs: Any, img_metas: Any) -> str:
        return "Forward function is replaced!"

    def load_state_dict(self, weights) -> AVAFastRCNN:
        pass


def _mock_sync_batchnorm(inputs):
    """Mock function for _sync_batch_to_normal_batch function.

    It returns its inputs
    """

    return inputs


@e2e_pytest_unit
def test_convert_sync_batch_to_normal_batch() -> None:
    """Test _convert_sync_batch_to_normal_batch function.

    <Steps>
        1. Create sample module, which has some Conv3D, SyncBatchNorm, BatchNorm3d ops
        2. Run _convert_sync_batch_to_normal_batch function to sample module
        3. Check SyncBatchNorm is changed into BatchNorm3d
        4. Check the other ops don't affect by this function
    """

    sample_module = nn.Sequential(
        nn.Conv3d(100, 100, 3), nn.SyncBatchNorm(100), nn.Conv3d(100, 100, 3), nn.BatchNorm3d(100)
    )
    output_module = _convert_sync_batch_to_normal_batch(sample_module)
    assert isinstance(output_module[0], nn.Conv3d)
    assert isinstance(output_module[1], nn.BatchNorm3d)
    assert isinstance(output_module[2], nn.Conv3d)
    assert isinstance(output_module[3], nn.BatchNorm3d)


class MockTaskProcessor:
    """Mock class of task_processor."""

    def __init__(self, model_cfg, deploy_cfg, device):
        self.model_cfg = model_cfg

    def init_pytorch_model(self, weights):
        if self.model_cfg.model == "cls":
            return MockRecognizer3D()
        return MockAVAFastRCNN()


def mock_build_task_processor(model_cfg, deploy_cfg, device):
    return MockTaskProcessor(model_cfg, deploy_cfg, device)


class TestExporter:
    """Test class for Exporter."""

    @e2e_pytest_unit
    def test_init(self, mocker) -> None:
        """Test __init__ function.

        <Steps>
            1. Create mock task_processor
            2. Create mock Recognizer3D using task_processor
            3. Get inputs
            4. Create mock AVAFastRCNN using task_processor
            5. Get inputs
            6. Check mo options when half precision
        """

        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.utils.export_utils.build_task_processor",
            side_effect=mock_build_task_processor,
        )

        recipe_cfg = Config(dict(model="cls"))
        deploy_cfg = Config(
            dict(
                backend_config=dict(
                    type="openvino",
                    mo_options={},
                    model_inputs=[dict(opt_shapes=dict(input=[1, 1, 3, 32, 224, 224]))],
                )
            )
        )
        exporter = Exporter(recipe_cfg, None, deploy_cfg, "./tmp_dir/openvino", False)
        assert isinstance(exporter.model, Recognizer3D)
        assert exporter.input_tensor.shape == torch.Size([1, 1, 3, 32, 224, 224])
        assert exporter.input_metas is None

        recipe_cfg = Config(dict(model="det"))
        deploy_cfg = Config(
            dict(
                backend_config=dict(
                    type="openvino",
                    mo_options={},
                    model_inputs=[dict(opt_shapes=dict(input=[1, 3, 32, 224, 224]))],
                )
            )
        )
        exporter = Exporter(recipe_cfg, None, deploy_cfg, "./tmp_dir/openvino", False)
        assert isinstance(exporter.model, AVAFastRCNN)
        assert exporter.input_tensor.shape == torch.Size([1, 3, 32, 224, 224])
        assert exporter.input_metas is not None

        exporter = Exporter(recipe_cfg, None, deploy_cfg, "./tmp_dir/openvino", True)
        assert exporter.deploy_cfg.backend_config.mo_options["flags"] == ["--compress_to_fp16"]

    @e2e_pytest_unit
    def test_export(self, mocker) -> None:
        """Test export function."""

        mocker.patch("otx.algorithms.action.adapters.mmaction.utils.export_utils.export", return_value=True)
        mocker.patch("otx.algorithms.action.adapters.mmaction.utils.export_utils.from_onnx", return_value=True)
        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.utils.export_utils.build_task_processor",
            side_effect=mock_build_task_processor,
        )

        recipe_cfg = Config(dict(model="cls"))
        deploy_cfg = Config(
            dict(
                backend_config=dict(
                    type="openvino",
                    mo_options={},
                    model_inputs=[dict(opt_shapes=dict(input=[1, 1, 3, 32, 224, 224]))],
                ),
                ir_config=dict(input_names=["input"], output_names=["output"]),
            )
        )
        exporter = Exporter(recipe_cfg, None, deploy_cfg, "./tmp_dir/openvino", False)
        exporter.export()



# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import pytest

from otx.api.entities.train_parameters import (
    TrainParameters,
    default_progress_callback,
    default_save_model_callback,
)
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestTrainParameters:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_train_parameters(self):
        """
        <b>Description:</b>
        Check TrainParameters class object attributes

        <b>Expected results:</b>
        Test passes if attributes of TrainParameters class object are equal to expected

        <b>Steps</b>
        1. Check attributes of TrainParameters object initialized with default parameters
        2. Check attributes of TrainParameters object initialized with "resume", "update_progress" and "save_model"
        parameters
        """
        # Checking attributes of TrainParameters object initiated with default parameters
        default_values_train_parameters = TrainParameters()
        assert not default_values_train_parameters.resume
        # Expected that update_progress equal to function for default parameters TrainParameters object
        assert default_values_train_parameters.update_progress == default_progress_callback
        # Expected that save_model is equal to function for default parameters TrainParameters object
        assert default_values_train_parameters.save_model == default_save_model_callback
        # Checking attributes of TrainParameters object initiated with specified
        progress_callback = default_progress_callback(99.9, 99.9)
        save_model_callback = default_save_model_callback()
        specified_values_train_parameters = TrainParameters(
            resume=True,
            update_progress=progress_callback,
            save_model=save_model_callback,
        )
        assert specified_values_train_parameters.resume
        assert specified_values_train_parameters.update_progress == progress_callback
        assert specified_values_train_parameters.save_model == save_model_callback



# Copyright (C) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

import pytest

from otx.api.entities.color import Color
from otx.api.entities.id import ID
from otx.api.entities.label import Domain, LabelEntity
from otx.api.utils.time_utils import now
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestDomain:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_domain(self):
        """
        <b>Description:</b>
        Check the Domain can correctly return the value

        <b>Expected results:</b>
        Test passes if the results match
        """
        domain = Domain
        assert len(domain) == 11


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestLabelEntity:

    creation_date = now()

    label_car_params = {
        "name": "car",
        "domain": Domain.DETECTION,
        "color": Color(255, 0, 0),
        "hotkey": "ctrl+1",
        "creation_date": creation_date,
        "is_empty": False,
        "id": ID(123456789),
    }

    label_person_params = {
        "name": "person",
        "domain": Domain.DETECTION,
        "color": Color(255, 17, 17),
        "hotkey": "ctrl+2",
        "creation_date": creation_date,
        "is_empty": False,
        "id": ID(987654321),
    }
    car = LabelEntity(**label_car_params)  # type: ignore
    empty = LabelEntity(name="empty", domain=Domain.SEGMENTATION, is_empty=True)
    person = LabelEntity(**label_person_params)  # type: ignore

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_label_entity(self):
        """
        <b>Description:</b>
        Check the LabelEntity can correctly return the value

        <b>Input data:</b>
        Dummy data

        <b>Expected results:</b>
        Test passes if incoming data is processed correctly

        <b>Steps</b>
        1. Use already created dummy data
        2. Check the processing of default values
        3. Check the processing of changed values
        """

        assert self.car == LabelEntity(**self.label_car_params)
        assert self.car != Domain
        assert self.car != self.person

        for attr in [
            "name",
            "domain",
            "color",
            "hotkey",
            "creation_date",
            "is_empty",
            "id",
        ]:
            assert getattr(self.car, attr) == self.label_car_params[attr]

        label_car_new_name = "electric car"
        label_car_new_domain = Domain.CLASSIFICATION
        label_car_new_color = Color(0, 255, 0)
        label_car_new_hotkey = "ctrl+2"
        label_car_new_id = ID(987654321)

        setattr(self.car, "name", label_car_new_name)
        setattr(self.car, "domain", label_car_new_domain)
        setattr(self.car, "color", label_car_new_color)
        setattr(self.car, "hotkey", label_car_new_hotkey)
        setattr(self.car, "id", label_car_new_id)

        assert self.car.name == label_car_new_name
        assert self.car.domain == label_car_new_domain
        assert self.car.color == label_car_new_color
        assert self.car.hotkey == label_car_new_hotkey
        assert self.car.id_ == label_car_new_id

        test_label_entity_repr = [
            f"{self.car.id_}",
            f"name={self.car.name}",
            f"hotkey={self.car.hotkey}",
            f"domain={self.car.domain}",
            f"color={self.car.color}",
        ]

        for i in test_label_entity_repr:
            assert i in self.car.__repr__()

        assert hash(self.car) == hash(str(self.car))
        assert self.car.__lt__(Domain) is False
        assert self.car.__gt__(Domain) is False

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_empty_label_entity(self):
        """
        <b>Description:</b>
        Check the LabelEntity can correctly return the value for empty label

        <b>Input data:</b>
        Dummy data

        <b>Expected results:</b>
        Test passes if incoming data is processed correctly

        <b>Steps</b>
        1. Use already created dummy data
        2. Check the processing of default values
        """

        assert self.empty.hotkey == ""
        assert self.empty.id_ == ID()
        assert type(self.empty.color) == Color

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_label_comparison(self):
        """
        <b>Description:</b>
        Check the LabelEntity __lt__, __gt__ methods with changed id

        <b>Input data:</b>
        Dummy data

        <b>Expected results:</b>
        Test passes if incoming data is processed correctly

        <b>Steps</b>
        1. Use already created dummy data
        2. Check the processing of changed id
        """

        self.empty.id_ = ID(999999999)
        assert self.empty > self.car
        assert self.car < self.empty



"""OTX Algorithms - Classification Heads."""

# Copyright (C) 2022 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

from .cls_head import ClsHead
from .contrastive_head import ConstrastiveHead
from .conv_head import ConvClsHead
from .custom_cls_head import CustomLinearClsHead, CustomNonLinearClsHead
from .custom_hierarchical_linear_cls_head import CustomHierarchicalLinearClsHead
from .custom_hierarchical_non_linear_cls_head import CustomHierarchicalNonLinearClsHead
from .custom_multi_label_linear_cls_head import CustomMultiLabelLinearClsHead
from .custom_multi_label_non_linear_cls_head import CustomMultiLabelNonLinearClsHead
from .mmov_cls_head import MMOVClsHead
from .non_linear_cls_head import NonLinearClsHead
from .semisl_cls_head import SemiLinearClsHead, SemiNonLinearClsHead
from .semisl_multilabel_cls_head import (
    SemiLinearMultilabelClsHead,
    SemiNonLinearMultilabelClsHead,
)
from .supcon_cls_head import SupConClsHead

__all__ = [
    "ConstrastiveHead",
    "CustomLinearClsHead",
    "CustomNonLinearClsHead",
    "CustomHierarchicalLinearClsHead",
    "CustomHierarchicalNonLinearClsHead",
    "CustomMultiLabelLinearClsHead",
    "CustomMultiLabelNonLinearClsHead",
    "SemiLinearMultilabelClsHead",
    "SemiNonLinearMultilabelClsHead",
    "NonLinearClsHead",
    "SemiLinearClsHead",
    "SemiNonLinearClsHead",
    "SupConClsHead",
    "MMOVClsHead",
    "ConvClsHead",
    "ClsHead",
]



# Copyright (C) 2021-2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#


import numpy as np
import openvino.runtime as ov


def create_ov_model():
    param = ov.opset10.parameter([1, 3, 32, 32], ov.Type.f32, name="in")
    constant = ov.opset10.constant(np.array([103.0, 116.0, 123.0]).reshape(1, 3, 1, 1), ov.Type.f32)
    node = ov.opset10.subtract(param, constant, "numpy")
    constant = ov.opset10.constant(np.random.normal(size=(16, 3, 3, 3)), ov.Type.f32)
    node = ov.opset10.convolution(node, constant, [2, 2], [1, 1], [1, 1], [1, 1], "explicit")
    constant = ov.opset10.constant(np.random.normal(size=(1, 16, 1, 1)), ov.Type.f32)
    node = ov.opset10.add(node, constant, "numpy")
    node = ov.opset10.clamp(node, 0, 6)
    node = ov.opset10.reduce_mean(node, [2, 3], False)
    node = ov.opset10.reshape(node, [-1, 16], False)
    constant = ov.opset10.constant(np.random.normal(size=(16, 10)), ov.Type.f32)
    node = ov.opset10.matmul(node, constant, False, False)
    result = ov.opset10.result(node, name="out")
    ov_model = ov.Model([result], [param], "model")
    return ov_model



# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import pytest
from attr import _make
from omegaconf import DictConfig

from otx.api.configuration.configurable_parameters import ConfigurableParameters
from otx.api.configuration.elements import ConfigurableEnum
from otx.api.configuration.elements.parameter_group import ParameterGroup
from otx.api.configuration.enums.config_element_type import ConfigElementType
from otx.api.configuration.enums.model_lifecycle import ModelLifecycle
from otx.api.configuration.helper.config_element_mapping import GroupElementMapping
from otx.api.configuration.helper.create import (
    construct_attrib_from_dict,
    construct_ui_rules_from_dict,
    contains_parameter_groups,
    create,
    create_default_configurable_enum_from_dict,
    create_nested_parameter_group,
    from_dict_attr,
    gather_parameter_arguments_and_values_from_dict,
)
from otx.api.configuration.ui_rules.rules import NullUIRules, Rule, UIRules
from otx.api.configuration.ui_rules.types import Action, Operator
from tests.unit.api.configuration.dummy_config import SomeEnumSelectable
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestCreateFunctions:
    @staticmethod
    def int_rule_dict():
        return {
            "parameter": "int parameter",
            "value": 100,
            "operator": Operator.LESS_THAN,
            "type": "RULE",
        }

    def int_ui_rules_params(self):
        return {"rules": [self.int_rule_dict()], "type": "UI_RULES"}

    def non_selectable_dict(self):
        return {
            "default_value": 70,
            "header": "non selectable parameter header",
            "ui_rules": self.int_ui_rules_params(),
            "type": "INTEGER",
        }

    @staticmethod
    def selectable_dict():
        return DictConfig(
            content={
                "header": "selectable parameter header",
                "enum_name": "test enum",
                "options": ["test_1", "test_2"],
                "default_value": 2,
                "type": "SELECTABLE",
                "value": 1,
                "affects_outcome_of": ModelLifecycle.TESTING,
            }
        )

    @staticmethod
    def check_ui_rules(
        ui_rules,
        expected_rules,
        expected_action=Action.DISABLE_EDITING,
        expected_operator=Operator.AND,
    ):
        assert isinstance(ui_rules, UIRules)
        assert ui_rules.rules == expected_rules
        assert ui_rules.action == expected_action
        assert ui_rules.operator == expected_operator
        assert ui_rules.type == ConfigElementType.UI_RULES

    def nested_config_dict_section(self):
        non_nested_parameters = {
            "header": "non-nested parameter header",
            "non_selectable": self.non_selectable_dict(),
            "type": GroupElementMapping.CONFIGURABLE_PARAMETERS,
        }
        selectable_parameters = {
            "header": "nested parameter header",
            "selectable": self.selectable_dict(),
            "type": GroupElementMapping.CONFIGURABLE_PARAMETERS,
        }

        nested_parameters = {
            "header": "nested parameters group header",
            "selectable": selectable_parameters,
            "type": GroupElementMapping.PARAMETER_GROUP,
        }

        return {
            "header": "test header",
            "non_nested": non_nested_parameters,
            "nested": nested_parameters,
            "type": GroupElementMapping.CONFIGURABLE_PARAMETERS,
        }

    @staticmethod
    def check_parameter_group(parameter_group, expected_type):
        # Checking parameter group attributes
        assert isinstance(parameter_group, expected_type)
        assert parameter_group.description == "Default parameter group description"
        assert parameter_group.groups == ["nested", "non_nested"]
        assert parameter_group.header == "test header"
        assert parameter_group.parameters == []
        assert parameter_group.type == ConfigElementType.CONFIGURABLE_PARAMETERS
        # Checking non-nested configurable parameter
        non_nested = parameter_group.non_nested
        assert isinstance(non_nested, ConfigurableParameters)
        assert non_nested.description == "Default parameter group description"
        assert non_nested.groups == []
        assert non_nested.header == "non-nested parameter header"
        assert non_nested.non_selectable == 70  # type: ignore
        assert non_nested.parameters == ["non_selectable"]
        assert non_nested.type == ConfigElementType.CONFIGURABLE_PARAMETERS
        # Checking nested parameter group
        nested = parameter_group.nested
        assert isinstance(nested, ParameterGroup)
        assert nested.groups == ["selectable"]
        assert nested.header == "nested parameters group header"
        assert nested.parameters == []
        assert nested.type == ConfigElementType.PARAMETER_GROUP
        # Checking nested parameter
        parameter = nested.selectable  # type: ignore
        assert isinstance(parameter, ConfigurableParameters)
        assert parameter.description == "Default parameter group description"
        assert parameter.groups == []
        assert parameter.header == "nested parameter header"
        assert parameter.selectable == 1  # type: ignore
        assert parameter.parameters == ["selectable"]
        assert parameter.type == ConfigElementType.CONFIGURABLE_PARAMETERS

    @staticmethod
    def check_non_nested_configurable_parameters(non_nested):
        assert isinstance(non_nested, ConfigurableParameters)
        assert non_nested.description == "Default parameter group description"
        assert non_nested.groups == []
        assert non_nested.header == "non-nested parameter header"
        assert non_nested.non_selectable == 70  # type: ignore
        assert non_nested.parameters == ["non_selectable"]
        assert non_nested.type == ConfigElementType.CONFIGURABLE_PARAMETERS

    @staticmethod
    def check_nested_parameter_group(nested):
        assert isinstance(nested, ParameterGroup)
        assert nested.groups == ["selectable"]
        assert nested.header == "nested parameters group header"
        assert nested.parameters == []
        assert nested.type == ConfigElementType.PARAMETER_GROUP
        # Checking nested parameter
        parameter = nested.selectable  # type: ignore
        assert isinstance(parameter, ConfigurableParameters)
        assert parameter.description == "Default parameter group description"
        assert parameter.groups == []
        assert parameter.header == "nested parameter header"
        assert parameter.selectable == 1  # type: ignore
        assert parameter.parameters == ["selectable"]
        assert parameter.type == ConfigElementType.CONFIGURABLE_PARAMETERS

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_construct_attrib_from_dict(self):
        """
        <b>Description:</b>
        Check "construct_attrib_from_dict" function

        <b>Input data:</b>
        "dict_object" dictionary or DictConfig parameter

        <b>Expected results:</b>
        Test passes if object returned by "construct_attrib_from_dict" function is equal to expected

        <b>Steps</b>
        1. Check Rule object returned by "construct_attrib_from_dict" function
        2. Check UIRules object returned by "construct_attrib_from_dict" function
        3. Check that ValueError exception is raised when unexpected "type" is specified in "dict_object" parameter
        """

        def check_rule(
            rule,
            expected_parameter,
            expected_value,
            expected_operator=Operator.EQUAL_TO,
        ):
            assert isinstance(rule, Rule)
            assert rule.parameter == expected_parameter
            assert rule.value == expected_value
            assert rule.type == ConfigElementType.RULE
            assert rule.operator == expected_operator

        # Checking Rule object returned by "construct_attrib_from_dict"
        # Constructing Rule with default optional parameters
        params = {
            "parameter": "string parameter",
            "value": "some string",
            "type": "RULE",
        }
        for dict_object in [params, DictConfig(content=params)]:
            check_rule(
                rule=construct_attrib_from_dict(dict_object=dict_object),
                expected_parameter="string parameter",
                expected_value="some string",
            )
        # Constructing Rule with specified optional parameters
        params = {
            "parameter": "int parameter",
            "value": 5,
            "operator": Operator.LESS_THAN,
            "type": "RULE",
        }
        for dict_object in [params, DictConfig(content=params)]:
            check_rule(
                rule=construct_attrib_from_dict(dict_object=dict_object),
                expected_parameter="int parameter",
                expected_value=5,
                expected_operator=Operator.LESS_THAN,
            )
        # Checking UIRules object returned by "construct_attrib_from_dict"
        rules = [
            {"parameter": "string parameter", "value": "some string"},
            {"parameter": "int parameter", "value": 2},
        ]
        # Constructing UIRules with default optional parameters
        params = {"rules": rules, "type": "UI_RULES"}
        for dict_object in [params, DictConfig(content=params)]:
            self.check_ui_rules(
                ui_rules=construct_attrib_from_dict(dict_object=dict_object),
                expected_rules=rules,
            )
        # Constructing UIRules with specified optional parameters
        params = {
            "rules": rules,
            "action": Action.ENABLE_EDITING,
            "operator": Operator.EQUAL_TO,
            "type": "UI_RULES",
        }
        for dict_object in [params, DictConfig(content=params)]:
            self.check_ui_rules(
                ui_rules=construct_attrib_from_dict(dict_object=dict_object),
                expected_rules=rules,
                expected_action=Action.ENABLE_EDITING,
                expected_operator=Operator.EQUAL_TO,
            )
        # Checking that ValueError exception is raised when unexpected "type" is specified in "dict_object"
        params = {
            "parameter": "string parameter",
            "value": "some string",
            "type": "unexpected type",
        }
        with pytest.raises(ValueError):
            construct_attrib_from_dict(dict_object=params)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_construct_ui_rules_from_dict(self):
        """
        <b>Description:</b>
        Check "construct_ui_rules_from_dict" function

        <b>Input data:</b>
        "ui_exposure_settings" dictionary or DictConfig parameter

        <b>Expected results:</b>
        Test passes if UIRules object returned by "construct_ui_rules_from_dict" function is equal to expected

        <b>Steps</b>
        1. Check UIRules object returned by "construct_ui_rules_from_dict" function when several "RULE" objects are
        specified in "rules" key of "ui_exposure_settings" parameter
        2. Check UIRules object returned by "construct_ui_rules_from_dict" function when several "RULE" objects and
        nested "UI_RULE" are specified in "rules" key of "ui_exposure_settings" parameter
        3. Check that NullUIRules object is returned by "construct_ui_rules_from_dict" function when None is specified
        as "ui_exposure_settings" parameter
        4. Check that NullUIRules object is returned by "construct_ui_rules_from_dict" function when empty list is
        specified in "rules" key of "ui_exposure_settings" parameter
        5. Check that ValueError exception is raised when one of objects specified in "rules" key of
        "ui_exposure_settings" parameter has unexpected "TYPE"
        """
        # Checking UIRules returned by "construct_ui_rules_from_dict" when several "RULE" are specified in "rules" of
        # "ui_exposure_settings"
        rule = self.int_rule_dict()
        other_rule = {"parameter": "bool parameter", "value": False, "type": "RULE"}
        params = {"rules": [rule, other_rule], "type": "UI_RULES"}
        expected_rules = [
            Rule(parameter="int parameter", value=100, operator=Operator.LESS_THAN),
            Rule(parameter="bool parameter", value=False),
        ]
        for ui_exposure_settings in [params, DictConfig(content=dict(params))]:
            self.check_ui_rules(
                ui_rules=construct_ui_rules_from_dict(ui_exposure_settings=ui_exposure_settings),
                expected_rules=expected_rules,
            )
        # Checking UIRules returned by "construct_ui_rules_from_dict" when several "RULE" and "UI_RULE" are specified in
        # "rules" of "ui_exposure_settings"
        nested_rule = {
            "parameter": "nested float parameter",
            "value": 10.6,
            "operator": Operator.GREATER_THAN,
            "type": "RULE",
        }

        other_nested_rule = {
            "parameter": "nested bool parameter",
            "value": True,
            "type": "RULE",
        }
        nested_ui_rules = {
            "rules": [nested_rule, other_nested_rule],
            "action": Action.ENABLE_EDITING,
            "operator": Operator.EQUAL_TO,
            "type": "UI_RULES",
        }
        params = {"rules": [rule, other_rule, nested_ui_rules], "type": "UI_RULES"}
        expected_nested_rules = [
            Rule(
                parameter="nested float parameter",
                value=10.6,
                operator=Operator.GREATER_THAN,
            ),
            Rule(parameter="nested bool parameter", value=True),
        ]
        expected_rules = [
            Rule(parameter="int parameter", value=100, operator=Operator.LESS_THAN),
            Rule(parameter="bool parameter", value=False),
            UIRules(
                rules=expected_nested_rules,
                action=Action.ENABLE_EDITING,
                operator=Operator.EQUAL_TO,
            ),
        ]
        for ui_exposure_settings in [params, DictConfig(content=dict(params))]:
            self.check_ui_rules(
                ui_rules=construct_ui_rules_from_dict(ui_exposure_settings=ui_exposure_settings),
                expected_rules=expected_rules,
            )
        # Checking that NullUIRules returned by "construct_ui_rules_from_dict" when None is specified as
        # "ui_exposure_settings"
        assert construct_ui_rules_from_dict(ui_exposure_settings=None) == NullUIRules()  # type: ignore
        # Checking that NullUIRules returned by "construct_ui_rules_from_dict" when empty list is specified in "rules"
        # key of "ui_exposure_settings"
        params = {"rules": [], "type": "UI_RULES"}
        assert construct_ui_rules_from_dict(ui_exposure_settings=params) == NullUIRules()
        # Checking that ValueError exception is raised when one of objects specified in "rules" key of
        # "ui_exposure_settings" has unexpected "TYPE"
        invalid_rule = {
            "parameter": "invalid rule",
            "value": False,
            "type": "unexpected type",
        }
        params = {"rules": [rule, invalid_rule], "type": "UI_RULES"}
        with pytest.raises(ValueError):
            construct_ui_rules_from_dict(params)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_create_default_configurable_enum_from_dict(self):
        """
        <b>Description:</b>
        Check "create_default_configurable_enum_from_dict" function

        <b>Input data:</b>
        "parameter_dict" dictionary or DictConfig parameter

        <b>Expected results:</b>
        Test passes if dictionary returned by "create_default_configurable_enum_from_dict" function is equal to expected

        <b>Steps</b>
        1. Check dictionary returned by "create_default_configurable_enum_from_dict" function when ConfigurableEnum
        element is specified as "default_value" key value of "parameter_dict" parameter
        2. Check dictionary returned by "create_default_configurable_enum_from_dict" function when int object is
        specified as "default_value" key value of "parameter_dict" parameter
        3. Check that TypeError exception is raised when DictConfig object with "content" attribute equal to None is
        specified as "default_value" key value of "parameter_dict" parameter
        """
        # Checking dictionary returned by "create_default_configurable_enum_from_dict" when ConfigurableEnum element is
        # specified as "default_value" key value of "parameter_dict"
        params = {
            "enum_name": "test enum",
            "options": ["test_1", "test_2"],
            "default_value": SomeEnumSelectable.TEST_NAME1,
        }
        for parameter_dict in [params, DictConfig(content=dict(params))]:
            configurable_enum = create_default_configurable_enum_from_dict(parameter_dict=parameter_dict)
            assert isinstance(configurable_enum, dict)
            assert len(configurable_enum) == 1
            default_value = configurable_enum.get("default_value")
            assert default_value.name == "TEST_NAME1"
            assert default_value.value == "test_name_1"
            assert type(default_value) == SomeEnumSelectable
        # Checking dictionary returned by "create_default_configurable_enum_from_dict" when int object is specified as
        # "default_value" key value of "parameter_dict"
        params = {
            "enum_name": "test enum",
            "options": ["test_1", "test_2"],
            "default_value": 2,
        }
        for parameter_dict in [params, DictConfig(content=dict(params))]:
            configurable_enum = create_default_configurable_enum_from_dict(parameter_dict=parameter_dict)
            assert isinstance(configurable_enum, dict)
            assert len(configurable_enum) == 1
            default_value = configurable_enum.get("default_value")
            assert default_value.name == "test_2"
            assert default_value.value == 2
            default_value_type = type(default_value)
            assert default_value_type.__name__ == "test enum"
            assert issubclass(default_value_type, ConfigurableEnum)
            assert len(default_value_type) == 2
            assert default_value_type.test_1.name == "test_1"
            assert default_value_type.test_1.value == 1
            assert default_value_type.test_2.name == "test_2"
            assert default_value_type.test_2.value == 2
        # Checking that TypeError exception is raised when DictConfig with "content" equal to None is specified as
        # "default_value" key value of "parameter_dict"
        with pytest.raises(TypeError):
            create_default_configurable_enum_from_dict(parameter_dict=DictConfig(content=None))

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_gather_parameter_arguments_and_values_from_dict(self):
        """
        <b>Description:</b>
        Check "gather_parameter_arguments_and_values_from_dict" function

        <b>Input data:</b>
        "config_dict_section" dictionary or DictConfig parameter

        <b>Expected results:</b>
        Test passes if dictionary returned by "gather_parameter_arguments_and_values_from_dict" function is equal to
        expected

        <b>Steps</b>
        1. Check dictionary returned by "gather_parameter_arguments_and_values_from_dict" function when INTEGER and
        SELECTABLE parameter dictionaries and non-dictionary value are specified in "config_dict_section" parameter
        2. Check that ValueError exception is raised when one of parameters contains "type" key equal to None
        """
        # Checking dictionary returned by "gather_parameter_arguments_and_values_from_dict" when INTEGER and
        # SELECTABLE argument dictionaries and non-dictionary value are specified in "config_dict_section"
        non_selectable_dict = self.non_selectable_dict()
        selectable_dict = self.selectable_dict()
        params = {
            "non_selectable": non_selectable_dict,
            "selectable": selectable_dict,
            "non_dict_key": 6,
            "type": "non required type",
        }
        for config_dict_section in [params, DictConfig(content=dict(params))]:
            arguments_and_values_dict = gather_parameter_arguments_and_values_from_dict(
                config_dict_section=config_dict_section
            )
            assert len(arguments_and_values_dict) == 3
            # Checking "make_arguments" key
            assert len(arguments_and_values_dict.get("make_arguments")) == 2
            # Checking non-Selectable argument
            argument = arguments_and_values_dict.get("make_arguments").get("non_selectable")
            assert isinstance(argument, _make._CountingAttr)
            assert argument.type == int
            metadata = argument.metadata
            assert metadata.get("default_value") == 70
            assert metadata.get("description") == "Default integer description"
            assert metadata.get("header") == "non selectable parameter header"
            assert metadata.get("affects_outcome_of") == ModelLifecycle.NONE
            assert metadata.get("type") == ConfigElementType.INTEGER
            assert metadata.get("ui_rules") == construct_ui_rules_from_dict(dict(self.int_ui_rules_params()))
            # Checking Selectable argument
            argument = arguments_and_values_dict.get("make_arguments").get("selectable")
            assert isinstance(argument, _make._CountingAttr)
            assert argument.type == ConfigurableEnum
            metadata = argument.metadata
            assert type(metadata.get("default_value")).__name__ == "test enum"
            assert metadata.get("default_value").name == "test_2"
            assert metadata.get("default_value").value == 2
            assert metadata.get("description") == "Default selectable description"
            assert metadata.get("header") == "selectable parameter header"
            assert metadata.get("affects_outcome_of") == ModelLifecycle.TESTING
            assert metadata.get("type") == ConfigElementType.SELECTABLE
            assert metadata.get("ui_rules") == NullUIRules()
            # Checking "call_arguments" key
            assert arguments_and_values_dict.get("call_arguments") == {"non_dict_key": 6}
            # Checking "values" key
            assert len(arguments_and_values_dict.get("values")) == 2
            assert arguments_and_values_dict.get("values").get("selectable") == 1
            assert not arguments_and_values_dict.get("values")["non_selectable"]
        # Checking that ValueError exception is raised when one of parameters contains "type" key equal to None
        non_selectable_dict.pop("type")
        config_dict_section = {
            "no_type_parameter": non_selectable_dict,
            "selectable": selectable_dict,
        }
        with pytest.raises(ValueError):
            gather_parameter_arguments_and_values_from_dict(config_dict_section=config_dict_section)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_contains_parameter_groups(self):
        """
        <b>Description:</b>
        Check "contains_parameter_groups" function

        <b>Input data:</b>
        "config_dict" dictionary or DictConfig parameter

        <b>Expected results:</b>
        Test passes if list returned by "contains_parameter_groups" function is equal to expected
        """
        configurable_parameters = {
            "header": "int group header",
            "int_value": 10,
            "type": GroupElementMapping.CONFIGURABLE_PARAMETERS,
        }
        parameter_group = {
            "header": "bool group header",
            "bool_value": False,
            "type": GroupElementMapping.PARAMETER_GROUP,
        }

        unexpected_type_group = {
            "header": "unexpected type header",
            "value": "some value",
            "type": "unexpected type",
        }

        parameters = {
            "configurable_parameters_test_group": configurable_parameters,
            "parameter_test_group": parameter_group,
            "unexpected_type_test_group": unexpected_type_group,
        }

        for config_dict in [parameters, DictConfig(content=dict(parameters))]:
            assert contains_parameter_groups(config_dict=config_dict) == [
                "configurable_parameters_test_group",
                "parameter_test_group",
            ]

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_create_nested_parameter_group(self):
        """
        <b>Description:</b>
        Check "create_nested_parameter_group" function

        <b>Input data:</b>
        "config_dict_section" dictionary or DictConfig parameter

        <b>Expected results:</b>
        Test passes if ParameterGroup object returned by "create_nested_parameter_group" function is equal to expected
        """
        parameters = self.nested_config_dict_section()
        for config_dict_section in parameters, DictConfig(content=dict(parameters)):
            nested_parameter_group = create_nested_parameter_group(config_dict_section=config_dict_section)
            self.check_parameter_group(
                parameter_group=nested_parameter_group,  # type: ignore
                expected_type=ParameterGroup,
            )

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_from_dict_attr(self):
        """
        <b>Description:</b>
        Check "from_dict_attr" function

        <b>Input data:</b>
        "config_dict" dictionary or DictConfig parameter

        <b>Expected results:</b>
        Test passes if ConfigurableParameters returned by "from_dict_attr" function is equal to expected
        """
        parameters = self.nested_config_dict_section()
        for config_dict in [parameters, DictConfig(content=dict(parameters))]:
            parameter_group = from_dict_attr(config_dict=config_dict)
            self.check_parameter_group(
                parameter_group=parameter_group,  # type: ignore
                expected_type=ConfigurableParameters,
            )

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_create(self):
        """
        <b>Description:</b>
        Check "create" function

        <b>Input data:</b>
        "input_config" dictionary or DictConfig parameter

        <b>Expected results:</b>
        Test passes if ConfigurableParameters object returned by "from_dict_attr" function is equal to expected

        <b>Steps</b>
        1. Check ConfigurableParameters object returned by "from_dict_attr" function when dictionary is specified as
        "input_config" parameter
        2. Check ConfigurableParameters object returned by "from_dict_attr" function when string is specified as
        "input_config" parameter
        """
        # Checking ConfigurableParameters returned by "from_dict_attr" when dictionary is specified as "input_config"
        parameters = self.nested_config_dict_section()
        for input_config in [parameters, DictConfig(content=dict(parameters))]:
            parameter_group = create(input_config=input_config)
            self.check_parameter_group(
                parameter_group=parameter_group,  # type: ignore
                expected_type=ConfigurableParameters,
            )

        # Checking ConfigurableParameters returned by "from_dict_attr" when string is specified as "input_config"
        str_parameter = {
            "header": "str parameter header",
            "enum_name": "test enum",
            "options": ["test_1", "test_2"],
            "default_value": 2,
            "type": "SELECTABLE",
            "value": 1,
            "affects_outcome_of": "TESTING",
        }
        str_parameter_group = {
            "header": "str parameter group header",
            "str_parameter": str_parameter,
            "type": "PARAMETER_GROUP",
        }
        parameter_group = create(str(str_parameter_group))
        assert parameter_group.description == "Default parameter group description"
        assert parameter_group.groups == []
        assert parameter_group.header == "str parameter group header"
        assert parameter_group.parameters == ["str_parameter"]
        assert parameter_group.str_parameter == 1  # type: ignore
        assert parameter_group.type == ConfigElementType.PARAMETER_GROUP



# Copyright (C) 2021-2023 Intel Corporation
# SPDX-Lggggcense-Identifier: Apache-2.0
#

import pytest
import torch
from torch.nn import functional as F

from otx.core.ov.ops.image_processings import InterpolateV4
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class TestInterpolateV4:
    @pytest.fixture(autouse=True)
    def setup(self):
        self.shape = (4, 3, 64, 64)
        self.input = torch.randn(self.shape)

    @e2e_pytest_unit
    def test_forward(self):

        with pytest.raises(ValueError):
            op = InterpolateV4(
                "dummy",
                shape=self.shape,
                mode="error",
                shape_calculation_mode="sizes",
            )

        with pytest.raises(ValueError):
            op = InterpolateV4(
                "dummy",
                shape=self.shape,
                mode="nearest",
                shape_calculation_mode="error",
            )

        with pytest.raises(ValueError):
            op = InterpolateV4(
                "dummy",
                shape=self.shape,
                mode="nearest",
                shape_calculation_mode="sizes",
                coordinate_transformation_mode="error",
            )

        with pytest.raises(ValueError):
            op = InterpolateV4(
                "dummy",
                shape=self.shape,
                mode="nearest",
                shape_calculation_mode="sizes",
                nearest_mode="error",
            )

        op = InterpolateV4(
            "dummy",
            shape=self.shape,
            mode="nearest",
            shape_calculation_mode="sizes",
        )
        assert torch.equal(
            op(self.input, torch.tensor((4, 3, 128, 128)), torch.tensor((1, 1, 2, 2))),
            F.interpolate(self.input, (128, 128), mode="nearest"),
        )

        op = InterpolateV4(
            "dummy",
            shape=self.shape,
            mode="nearest",
            shape_calculation_mode="scales",
        )
        assert torch.equal(
            op(self.input, torch.tensor((4, 3, 128, 128)), torch.tensor((1, 1, 2, 2))),
            F.interpolate(self.input, scale_factor=(2, 2), mode="nearest"),
        )

        op = InterpolateV4(
            "dummy",
            shape=self.shape,
            mode="linear",
            shape_calculation_mode="sizes",
        )
        assert torch.equal(
            op(self.input, torch.tensor((4, 3, 128, 128)), torch.tensor((1, 1, 2, 2))),
            F.interpolate(self.input, (128, 128), mode="bilinear", align_corners=False),
        )

        op = InterpolateV4(
            "dummy",
            shape=self.shape,
            mode="cubic",
            shape_calculation_mode="sizes",
        )
        assert torch.equal(
            op(self.input, torch.tensor((4, 3, 128, 128)), torch.tensor((1, 1, 2, 2))),
            F.interpolate(self.input, (128, 128), mode="bicubic", align_corners=False),
        )



"""Collections of Utils for common OTX algorithms."""

# Copyright (C) 2022 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

import importlib
import inspect
import os
import random
from collections import defaultdict
from typing import Callable, Optional, Tuple

import numpy as np
import yaml


class UncopiableDefaultDict(defaultdict):
    """Defauldict type object to avoid deepcopy."""

    def __deepcopy__(self, memo):
        """Deepcopy."""
        return self


def load_template(path):
    """Loading model template function."""
    with open(path, encoding="UTF-8") as f:
        template = yaml.safe_load(f)
    return template


def get_task_class(path: str):
    """Return Task classes."""
    module_name, class_name = path.rsplit(".", 1)
    module = importlib.import_module(module_name)
    return getattr(module, class_name)


def get_arg_spec(  # noqa: C901  # pylint: disable=too-many-branches
    fn: Callable,  # pylint: disable=invalid-name
    depth: Optional[int] = None,
) -> Tuple[str, ...]:
    """Get argument spec of function."""

    args = set()

    cls_obj = None
    if inspect.ismethod(fn):
        fn_name = fn.__name__
        cls_obj = fn.__self__
        if not inspect.isclass(cls_obj):
            cls_obj = cls_obj.__class__
    else:
        fn_name = fn.__name__
        names = fn.__qualname__.split(".")
        if len(names) > 1 and names[-1] == fn_name:
            cls_obj = globals()[".".join(names[:-1])]

    if cls_obj:
        for obj in cls_obj.mro():  # type: ignore
            fn_obj = cls_obj.__dict__.get(fn_name, None)
            if fn_obj is not None:
                if isinstance(fn_obj, staticmethod):
                    cls_obj = None
                    break

    if cls_obj is None:
        # function, staticmethod
        spec = inspect.getfullargspec(fn)
        args.update(spec.args)
    else:
        # method, classmethod
        for i, obj in enumerate(cls_obj.mro()):  # type: ignore
            if depth is not None and i == depth:
                break
            method = getattr(obj, fn_name, None)
            if method is None:
                break
            spec = inspect.getfullargspec(method)
            args.update(spec.args[1:])
            if spec.varkw is None and spec.varargs is None:
                break
    return tuple(args)


def set_random_seed(seed, logger, deterministic=False):
    """Set random seed.

    Args:
        seed (int): Seed to be used.
        logger (logging.Logger): logger for logging seed info
        deterministic (bool): Whether to set the deterministic option for
            CUDNN backend, i.e., set `torch.backends.cudnn.deterministic`
            to True and `torch.backends.cudnn.benchmark` to False.
            Default: False.
    """
    import torch

    random.seed(seed)
    np.random.seed(seed)
    torch.manual_seed(seed)
    torch.cuda.manual_seed_all(seed)
    os.environ["PYTHONHASHSEED"] = str(seed)
    logger.info(f"Training seed was set to {seed} w/ deterministic={deterministic}.")
    if deterministic:
        torch.backends.cudnn.deterministic = True
        torch.backends.cudnn.benchmark = False



"""Base Action classification mmdeply cfg."""

ir_config = dict(
    type="onnx",
    export_params=True,
    keep_initializers_as_inputs=False,
    opset_version=11,
    save_file="end2end.onnx",
    input_names=["data"],
    output_names=["logits"],
    input_shape=None,
    optimize=False,
)
codebase_config = dict(type="mmaction", task="VideoRecognition")
backend_config = dict(
    type="openvino",
    mo_options=dict(args=dict({"--source_layout": "?bctwh"})),
    model_inputs=[dict(opt_shapes=dict(input=[1, 1, 3, 32, 224, 224]))],
)



# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#


from datetime import datetime

import numpy as np
import pytest

from otx.api.configuration import ConfigurableParameters
from otx.api.entities.annotation import (
    Annotation,
    AnnotationSceneEntity,
    AnnotationSceneKind,
)
from otx.api.entities.color import Color
from otx.api.entities.dataset_item import DatasetItemEntity
from otx.api.entities.datasets import DatasetEntity
from otx.api.entities.id import ID
from otx.api.entities.image import Image
from otx.api.entities.label import Domain, LabelEntity
from otx.api.entities.label_schema import LabelGroup, LabelSchemaEntity
from otx.api.entities.metrics import (
    BarChartInfo,
    BarMetricsGroup,
    ColorPalette,
    MatrixMetric,
    Performance,
    ScoreMetric,
    VisualizationType,
)
from otx.api.entities.model import ModelConfiguration, ModelEntity
from otx.api.entities.resultset import ResultSetEntity
from otx.api.entities.scored_label import ScoredLabel
from otx.api.entities.shapes.rectangle import Rectangle
from otx.api.usecases.evaluation.accuracy import (
    Accuracy,
    compute_unnormalized_confusion_matrices_from_resultset,
    precision_metrics_group,
    recall_metrics_group,
)
from otx.api.usecases.evaluation.averaging import MetricAverageMethod
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


class CommonActions:
    color = Color(0, 255, 0)
    creation_date = datetime(year=2021, month=12, day=27)
    image = Image(data=np.random.randint(low=0, high=255, size=(10, 16, 3)))

    def car(self) -> LabelEntity:
        return LabelEntity(
            name="car",
            domain=Domain.DETECTION,
            color=self.color,
            creation_date=self.creation_date,
            id=ID("car_label"),
        )

    def human(self) -> LabelEntity:
        return LabelEntity(
            name="human",
            domain=Domain.DETECTION,
            color=self.color,
            creation_date=self.creation_date,
            id=ID("human_label"),
        )

    def dog(self) -> LabelEntity:
        return LabelEntity(
            name="dog",
            domain=Domain.DETECTION,
            color=self.color,
            creation_date=self.creation_date,
            id=ID("dog_label"),
        )

    def human_1_dataset_item(self) -> DatasetItemEntity:
        human_1_roi = Annotation(shape=Rectangle.generate_full_box(), labels=[ScoredLabel(self.human())])
        human_1_annotation = Annotation(
            shape=Rectangle(x1=0.3, y1=0, x2=0.4, y2=0.2),
            labels=[ScoredLabel(self.human())],
        )
        human_1_annotation_scene = AnnotationSceneEntity(
            annotations=[human_1_annotation], kind=AnnotationSceneKind.ANNOTATION
        )
        human_1_dataset_item = DatasetItemEntity(
            media=self.image, annotation_scene=human_1_annotation_scene, roi=human_1_roi
        )
        return human_1_dataset_item

    def human_2_dataset_item(self) -> DatasetItemEntity:
        human_2_roi = Annotation(shape=Rectangle.generate_full_box(), labels=[ScoredLabel(self.human())])
        human_2_annotation = Annotation(
            shape=Rectangle(x1=0.6, y1=0, x2=0.7, y2=0.3),
            labels=[ScoredLabel(self.human())],
        )
        human_2_annotation_scene = AnnotationSceneEntity(
            annotations=[human_2_annotation], kind=AnnotationSceneKind.ANNOTATION
        )
        human_2_dataset_item = DatasetItemEntity(
            media=self.image, annotation_scene=human_2_annotation_scene, roi=human_2_roi
        )
        return human_2_dataset_item

    def human_3_dataset_item(self) -> DatasetItemEntity:
        human_3_roi = Annotation(shape=Rectangle.generate_full_box(), labels=[ScoredLabel(self.human())])
        human_3_annotation = Annotation(
            shape=Rectangle(x1=0.7, y1=0, x2=0.8, y2=0.3),
            labels=[ScoredLabel(self.human())],
        )
        human_3_annotation_scene = AnnotationSceneEntity(
            annotations=[human_3_annotation], kind=AnnotationSceneKind.ANNOTATION
        )
        human_3_dataset_item = DatasetItemEntity(
            media=self.image, annotation_scene=human_3_annotation_scene, roi=human_3_roi
        )
        return human_3_dataset_item

    def result_set(self) -> ResultSetEntity:
        configurable_params = ConfigurableParameters(header="Test model configurable params")
        labels_group = LabelGroup(
            name="model_labels_group",
            labels=[self.car(), self.human(), self.dog()],
        )
        other_label_group = LabelGroup(
            name="other_model_labels_group",
            labels=[self.human(), self.dog()],
        )
        model_configuration = ModelConfiguration(
            configurable_params,
            LabelSchemaEntity(label_groups=[labels_group, other_label_group]),
        )
        model = ModelEntity(train_dataset=DatasetEntity(), configuration=model_configuration)

        car_1_roi = Annotation(
            shape=Rectangle.generate_full_box(),
            labels=[ScoredLabel(self.car())],
        )
        car_1_annotation = Annotation(
            shape=Rectangle(x1=0, y1=0, x2=0.3, y2=0.2),
            labels=[ScoredLabel(self.car())],
        )
        car_1_annotation_scene = AnnotationSceneEntity(
            annotations=[car_1_annotation], kind=AnnotationSceneKind.ANNOTATION
        )
        car_1_dataset_item = DatasetItemEntity(media=self.image, annotation_scene=car_1_annotation_scene, roi=car_1_roi)

        car_2_roi = Annotation(shape=Rectangle.generate_full_box(), labels=[ScoredLabel(self.car())])
        car_2_annotation = Annotation(
            shape=Rectangle(x1=0.8, y1=0, x2=1.0, y2=0.2),
            labels=[ScoredLabel(self.car())],
        )
        car_2_annotation_scene = AnnotationSceneEntity(
            annotations=[car_2_annotation], kind=AnnotationSceneKind.ANNOTATION
        )
        car_2_dataset_item = DatasetItemEntity(media=self.image, annotation_scene=car_2_annotation_scene, roi=car_2_roi)

        dog_1_roi = Annotation(shape=Rectangle.generate_full_box(), labels=[ScoredLabel(self.dog())])
        dog_1_annotation = Annotation(
            shape=Rectangle(x1=0.5, y1=0, x2=0.6, y2=0.1),
            labels=[ScoredLabel(self.dog())],
        )
        dog_1_annotation_scene = AnnotationSceneEntity(
            annotations=[dog_1_annotation], kind=AnnotationSceneKind.ANNOTATION
        )
        dog_1_dataset_item = DatasetItemEntity(media=self.image, annotation_scene=dog_1_annotation_scene, roi=dog_1_roi)

        dog_2_roi = Annotation(shape=Rectangle.generate_full_box(), labels=[ScoredLabel(self.dog())])
        dog_2_annotation = Annotation(
            shape=Rectangle(x1=0.7, y1=0, x2=0.8, y2=0.3),
            labels=[ScoredLabel(self.dog())],
        )
        dog_2_annotation_scene = AnnotationSceneEntity(
            annotations=[dog_2_annotation], kind=AnnotationSceneKind.ANNOTATION
        )
        dog_2_dataset_item = DatasetItemEntity(media=self.image, annotation_scene=dog_2_annotation_scene, roi=dog_2_roi)

        ground_truth_dataset = DatasetEntity(
            [
                car_1_dataset_item,
                car_2_dataset_item,
                self.human_1_dataset_item(),
                self.human_2_dataset_item(),
                self.human_3_dataset_item(),
                dog_1_dataset_item,
            ]
        )
        prediction_dataset = DatasetEntity(
            [
                car_1_dataset_item,
                car_2_dataset_item,
                self.human_1_dataset_item(),
                self.human_2_dataset_item(),
                dog_1_dataset_item,
                dog_2_dataset_item,
            ]
        )
        result_set = ResultSetEntity(
            model=model,
            ground_truth_dataset=ground_truth_dataset,
            prediction_dataset=prediction_dataset,
        )
        return result_set

    def single_label_result_set(self) -> ResultSetEntity:
        configurable_params = ConfigurableParameters(header="Test model configurable params")
        labels_group = LabelGroup(
            name="single_class_model_labels_group",
            labels=[self.human()],
        )
        model_configuration = ModelConfiguration(configurable_params, LabelSchemaEntity(label_groups=[labels_group]))
        model = ModelEntity(train_dataset=DatasetEntity(), configuration=model_configuration)

        ground_truth_dataset = DatasetEntity(
            [
                self.human_1_dataset_item(),
                self.human_2_dataset_item(),
                self.human_3_dataset_item(),
            ]
        )
        prediction_dataset = DatasetEntity([self.human_1_dataset_item(), self.human_2_dataset_item()])
        result_set = ResultSetEntity(
            model=model,
            ground_truth_dataset=ground_truth_dataset,
            prediction_dataset=prediction_dataset,
        )
        return result_set

    @staticmethod
    def check_confusion_matrix(matrix, expected_name, expected_labels, expected_matrix) -> None:
        assert matrix.name == expected_name
        assert matrix.row_labels == matrix.column_labels
        assert matrix.row_labels == expected_labels
        assert np.array_equal(matrix.matrix_values, expected_matrix)


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestAccuracyFunctions:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_precision_metrics_group(self):
        """
        <b>Description:</b>
        Check "precision_metrics_group" function

        <b>Input data:</b>
        "confusion_matrix" MatrixMetric-class object with specified "name", "matrix_values", "row_labels",
        "column_labels" and "normalize" parameters

        <b>Expected results:</b>
        Test passes if "BarMetricsGroup" object returned by "precision_metrics_group" function is equal to expected

        <b>Steps</b>
        1. Check "BarMetricsGroup" object returned by "precision_metrics_group" function for "confusion_matrix" with
        default value of "row_labels" parameter
        2. Check "BarMetricsGroup" object returned by "precision_metrics_group" function for "confusion_matrix" with
        specified value of "row_labels" parameter
        """

        def check_precision_metrics_group(matrix_for_precision, expected_metrics):
            precision_metrics = precision_metrics_group(matrix_for_precision)
            assert isinstance(precision_metrics, BarMetricsGroup)
            assert precision_metrics.metrics == expected_metrics
            assert isinstance(precision_metrics.visualization_info, BarChartInfo)
            assert precision_metrics.visualization_info.name == "Precision per class"
            assert precision_metrics.visualization_info.palette == ColorPalette.LABEL
            assert precision_metrics.visualization_info.type == VisualizationType.BAR

        matrix_values = np.array([[0, 0.5, 0.5], [0, 0.5, 0.5], [1, 0, 0]])
        # Checking "BarMetricsGroup" object returned by "precision_metrics_group" for "confusion_matrix" with default
        # "row_labels"
        confusion_matrix = MatrixMetric(name="no_row_labels MatrixMetric", matrix_values=matrix_values)
        check_precision_metrics_group(
            matrix_for_precision=confusion_matrix,
            expected_metrics=[
                ScoreMetric(name=np.int32(0), value=0.0),
                ScoreMetric(name=np.int32(1), value=0.5),
                ScoreMetric(name=np.int32(2), value=0.0),
            ],
        )
        # Checking "BarMetricsGroup" object returned by "precision_metrics_group" for "confusion_matrix" with specified
        # "row_labels"
        confusion_matrix = MatrixMetric(
            name="row_labels MatrixMetric",
            matrix_values=matrix_values,
            row_labels=["label for row_1", "label for row_2", "label for row_3"],
            column_labels=[
                "label for column_1",
                "label for column_2",
                "label for column_3",
            ],
        )
        check_precision_metrics_group(
            matrix_for_precision=confusion_matrix,
            expected_metrics=[
                ScoreMetric(name="label for row_1", value=0.0),
                ScoreMetric(name="label for row_2", value=0.5),
                ScoreMetric(name="label for row_3", value=0.0),
            ],
        )

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_recall_metrics_group(self):
        """
        <b>Description:</b>
        Check "recall_metrics_group" function

        <b>Input data:</b>
        MatrixMetric-class object with specified "name", "matrix_values", "row_labels", "column_labels" and "normalize"
        parameters

        <b>Expected results:</b>
        Test passes if "BarMetricsGroup" object returned by "recall_metrics_group" function is equal to expected

        <b>Steps</b>
        1. Check "BarMetricsGroup" object returned by "recall_metrics_group" function when "confusion_matrix"
        initialized with default "row_labels" parameter
        2. Check "BarMetricsGroup" object returned by "recall_metrics_group" function when "confusion_matrix"
        initialized with specified "row_labels" parameter
        """

        def check_recall_metrics_group(matrix_for_recall, expected_metrics):
            recall_metrics = recall_metrics_group(matrix_for_recall)
            assert isinstance(recall_metrics, BarMetricsGroup)
            assert recall_metrics.metrics == expected_metrics
            assert isinstance(recall_metrics.visualization_info, BarChartInfo)
            assert recall_metrics.visualization_info.name == "Recall per class"
            assert recall_metrics.visualization_info.palette == ColorPalette.LABEL
            assert recall_metrics.visualization_info.type == VisualizationType.BAR

        matrix_values = np.array([[0, 0.8, 0.4], [0, 0.8, 0.2], [0.4, 0.2, 0]])
        # Checking "BarMetricsGroup" returned by "recall_metrics_group" when "confusion_matrix" initialized with
        # default "row_labels"
        confusion_matrix = MatrixMetric(name="no_row_labels MatrixMetric", matrix_values=matrix_values)

        check_recall_metrics_group(
            matrix_for_recall=confusion_matrix,
            expected_metrics=[
                ScoreMetric(name=np.int32(0), value=0.0),
                ScoreMetric(name=np.int32(1), value=0.8),
                ScoreMetric(name=np.int32(2), value=0.0),
            ],
        )
        # Checking "BarMetricsGroup" returned by "recall_metrics_group" when "confusion_matrix" initialized with
        # specified "row_labels"
        confusion_matrix = MatrixMetric(
            name="row_labels MatrixMetric",
            matrix_values=matrix_values,
            row_labels=["label for row_1", "label for row_2", "label for row_3"],
            column_labels=[
                "label for column_1",
                "label for column_2",
                "label for column_3",
            ],
        )
        check_recall_metrics_group(
            matrix_for_recall=confusion_matrix,
            expected_metrics=[
                ScoreMetric(name="label for row_1", value=0.0),
                ScoreMetric(name="label for row_2", value=0.8),
                ScoreMetric(name="label for row_3", value=0.0),
            ],
        )

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_compute_unnormalized_confusion_matrices_from_resultset(self):
        """
        <b>Description:</b>
        Check "compute_unnormalized_confusion_matrices_from_resultset" function

        <b>Input data:</b>
        ResultSetEntity-class object with specified "ground_truth_dataset" and "prediction_dataset" parameters

        <b>Expected results:</b>
        Test passes if list returned by "compute_unnormalized_confusion_matrices_from_resultset" function is equal to
        expected

        <b>Steps</b>
        1. Check list returned by "compute_unnormalized_confusion_matrices_from_resultset" function for model with
        multiple labels specified in LabelGroup
        2. Check list returned by "compute_unnormalized_confusion_matrices_from_resultset" function for model with
        single label specified in LabelGroup
        3. Check "ValueError" exception is raised when trying to compute confusion matrix for "ResultSetEntity" object
        with "ground_truth_dataset" attribute equal to empty list
        4. Check "ValueError" exception is raised when trying to compute confusion matrix for "ResultSetEntity" object
        with "prediction_dataset" attribute equal to empty list
        """
        # Checking list returned by "compute_unnormalized_confusion_matrices_from_resultset" for model with multiple
        # labels in LabelGroup
        result_set = CommonActions().result_set()
        confusion_matrices = compute_unnormalized_confusion_matrices_from_resultset(result_set)
        assert len(confusion_matrices) == 2
        # Checking first confusion matrix
        confusion_matrix = confusion_matrices[0]
        CommonActions.check_confusion_matrix(
            matrix=confusion_matrix,
            expected_name="model_labels_group",
            expected_labels=["car", "dog", "human"],
            expected_matrix=np.array([[2, 0, 0], [0, 1, 0], [0, 1, 2]]),
        )
        # Checking second confusion matrix
        confusion_matrix = confusion_matrices[1]
        CommonActions.check_confusion_matrix(
            matrix=confusion_matrix,
            expected_name="other_model_labels_group",
            expected_labels=["dog", "human"],
            expected_matrix=np.array([[1, 0], [1, 2]]),
        )
        # Checking list returned by "compute_unnormalized_confusion_matrices_from_resultset" function for model with
        # single label specified in LabelGroup
        result_set = CommonActions().single_label_result_set()
        confusion_matrices = compute_unnormalized_confusion_matrices_from_resultset(result_set)
        assert len(confusion_matrices) == 1
        confusion_matrix = confusion_matrices[0]
        CommonActions.check_confusion_matrix(
            matrix=confusion_matrix,
            expected_name="single_class_model_labels_group",
            expected_labels=["human", "~ human"],
            expected_matrix=np.array([[2, 0], [0, 0]]),
        )
        # Checking "ValueError" exception is raised when trying to compute confusion matrix for "ResultSetEntity" with
        # "ground_truth_dataset" equal to empty list
        result_set.ground_truth_dataset = []
        with pytest.raises(ValueError):
            compute_unnormalized_confusion_matrices_from_resultset(result_set)
        # Checking "ValueError" exception is raised when trying to compute confusion matrix for "ResultSetEntity" with
        # "prediction_dataset" equal to empty list
        result_set = CommonActions().single_label_result_set()
        result_set.prediction_dataset = []
        with pytest.raises(ValueError):
            compute_unnormalized_confusion_matrices_from_resultset(result_set)


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestAccuracy:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_accuracy_initialization(self):
        """
        <b>Description:</b>
        Check "Accuracy" class object initialization

        <b>Input data:</b>
        "Accuracy" class object with specified "resultset" and "average" parameters

        <b>Expected results:</b>
        Test passes if attributes of initialized "Accuracy" class object are equal to expected

        <b>Steps</b>
        1. Check attributes of "Accuracy" class object initialized with default optional parameters
        2. Check attributes of "Accuracy" class object initialized with specified optional parameters
        """

        def check_confusion_matrices(unnormalized_matrices):
            assert len(unnormalized_matrices) == 2
            # Checking first confusion matrix
            confusion_matrix = unnormalized_matrices[0]
            assert isinstance(confusion_matrix, MatrixMetric)
            CommonActions.check_confusion_matrix(
                matrix=confusion_matrix,
                expected_name="model_labels_group",
                expected_labels=["car", "dog", "human"],
                expected_matrix=np.array([[2, 0, 0], [0, 1, 0], [0, 1, 2]]),
            )
            # Checking second confusion matrix
            confusion_matrix = unnormalized_matrices[1]
            assert isinstance(confusion_matrix, MatrixMetric)
            CommonActions.check_confusion_matrix(
                matrix=confusion_matrix,
                expected_name="other_model_labels_group",
                expected_labels=["dog", "human"],
                expected_matrix=np.array([[1, 0], [1, 2]]),
            )

        result_set = CommonActions().result_set()
        # Checking attributes of "Accuracy" object initialized with default optional parameters
        accuracy = Accuracy(result_set)
        # Checking "accuracy" attribute
        assert accuracy.accuracy == ScoreMetric(name="Accuracy", value=0.8)
        # Checking "unnormalized_matrices" attribute
        actual_unnormalized_matrices = accuracy._unnormalized_matrices
        check_confusion_matrices(actual_unnormalized_matrices)
        # Checking attributes of "Accuracy" object initialized with specified optional parameters
        accuracy = Accuracy(resultset=result_set, average=MetricAverageMethod.MACRO)
        # Checking "accuracy" attribute
        assert accuracy.accuracy == ScoreMetric(name="Accuracy", value=0.7916666666666667)
        # Checking "unnormalized_matrices" attribute
        actual_unnormalized_matrices = accuracy._unnormalized_matrices
        check_confusion_matrices(actual_unnormalized_matrices)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_accuracy_get_performance(self):
        """
        <b>Description:</b>
        Check "Accuracy" class object "get_performance" method

        <b>Input data:</b>
        "Accuracy" class object with specified "resultset" and "average" parameters

        <b>Expected results:</b>
        Test passes if "Performance" object returned by "get_performance" method is equal to expected
        """

        def check_precision_recall_metrics(precision_metric, recall_metric, expected_precision, expected_recall):
            # Checking Precision per class metric
            assert isinstance(precision_metric, BarMetricsGroup)
            assert precision_metric.metrics == expected_precision
            assert precision_metric.visualization_info.name == "Precision per class"
            assert precision_metric.visualization_info.palette == ColorPalette.LABEL
            assert precision_metric.visualization_info.type == VisualizationType.BAR
            # Checking Recall per class metric
            assert isinstance(recall_metric, BarMetricsGroup)
            assert recall_metric.metrics == expected_recall
            assert recall_metric.visualization_info.name == "Recall per class"
            assert recall_metric.visualization_info.palette == ColorPalette.LABEL

        result_set = CommonActions().result_set()
        accuracy = Accuracy(result_set)
        actual_performance = accuracy.get_performance()
        assert isinstance(actual_performance, Performance)
        assert actual_performance.score == ScoreMetric(name="Accuracy", value=0.8)
        assert len(actual_performance.dashboard_metrics) == 5
        # Checking dashboard_metrics
        # Checking first MatrixMetric object
        actual_matrix_metric = actual_performance.dashboard_metrics[0].metrics[0]
        CommonActions.check_confusion_matrix(
            matrix=actual_matrix_metric,
            expected_name="model_labels_group",
            expected_labels=["car", "dog", "human"],
            expected_matrix=np.array([[1, 0, 0], [0, 1, 0], [0, 0.33333334, 0.6666667]], dtype=np.float32),
        )
        # Checking second MatrixMetric object
        actual_matrix_metric = actual_performance.dashboard_metrics[0].metrics[1]
        CommonActions.check_confusion_matrix(
            matrix=actual_matrix_metric,
            expected_name="other_model_labels_group",
            expected_labels=["dog", "human"],
            expected_matrix=np.array([[1, 0], [0.33333334, 0.6666667]], dtype=np.float32),
        )
        # Checking Precision and Recall BarMetricsGroup for first label_group
        precision_per_class_metric = actual_performance.dashboard_metrics[1]
        recall_per_class_metric = actual_performance.dashboard_metrics[2]
        check_precision_recall_metrics(
            precision_metric=precision_per_class_metric,
            recall_metric=recall_per_class_metric,
            expected_precision=[
                ScoreMetric("car", 1.0),
                ScoreMetric("dog", 0.5),
                ScoreMetric("human", 1.0),
            ],
            expected_recall=[
                ScoreMetric("car", 1.0),
                ScoreMetric("dog", 1.0),
                ScoreMetric("human", 0.6666666666666666),
            ],
        )
        # Checking Precision and Recall BarMetricsGroup for second label_group
        precision_per_class_metric = actual_performance.dashboard_metrics[3]
        recall_per_class_metric = actual_performance.dashboard_metrics[4]
        check_precision_recall_metrics(
            precision_metric=precision_per_class_metric,
            recall_metric=recall_per_class_metric,
            expected_precision=[ScoreMetric("dog", 0.5), ScoreMetric("human", 1.0)],
            expected_recall=[
                ScoreMetric("dog", 1.0),
                ScoreMetric("human", 0.6666666666666666),
            ],
        )

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_accuracy_compute_accuracy(self):
        """
        <b>Description:</b>
        Check "Accuracy" class object "_compute_accuracy" method

        <b>Input data:</b>
        "confusion_matrices" list and "average" MetricAverageMethod parameters

        <b>Expected results:</b>
        Test passes if value returned by "_compute_accuracy" method is equal to expected

        <b>Steps</b>
        1. Check value returned by "_compute_accuracy" method for single confusion matrix specified as
        "confusion_matrices" parameter
        2. Check value returned by "_compute_accuracy" method for several confusion matrices specified as
        "confusion_matrices" parameter
        3. Check "ValueError" exception is raised when empty list is specified as "confusion_matrices" parameter
        4. Check "RuntimeError" exception is raised when unexpected method is specified as "average" parameter
        """
        result_set = CommonActions().single_label_result_set()
        accuracy = Accuracy(result_set)
        # Checking value returned by "_compute_accuracy" for single confusion matrix specified as "confusion_matrices"
        confusion_matrix = MatrixMetric(
            name="confusion_matrix",
            matrix_values=np.array([[6, 1, 0], [2, 6, 1], [0, 0, 4]]),
        )
        expected_accuracy = np.float64(0.8)
        assert (
            accuracy._compute_accuracy(average=MetricAverageMethod.MICRO, confusion_matrices=[confusion_matrix])
            == expected_accuracy
        )
        assert (
            accuracy._compute_accuracy(average=MetricAverageMethod.MACRO, confusion_matrices=[confusion_matrix])
            == expected_accuracy
        )
        # Checking value returned by "_compute_accuracy" for several confusion matrices specified as
        # "confusion_matrices"
        other_confusion_matrix = MatrixMetric(
            name="other_confusion_matrix",
            matrix_values=np.array([[4, 0, 0], [2, 4, 0], [0, 0, 6]]),
        )
        assert accuracy._compute_accuracy(
            average=MetricAverageMethod.MICRO,
            confusion_matrices=[confusion_matrix, other_confusion_matrix],
        ) == np.float64(0.8333333333333334)
        assert accuracy._compute_accuracy(
            average=MetricAverageMethod.MACRO,
            confusion_matrices=[confusion_matrix, other_confusion_matrix],
        ) == np.float64(0.8375)
        # Checking "ValueError" exception is raised when empty list is specified as "confusion_matrices"
        with pytest.raises(ValueError):
            accuracy._compute_accuracy(average=MetricAverageMethod.MACRO, confusion_matrices=[])
        # Checking "RuntimeError" exception is raised when unexpected method is specified as "average"
        with pytest.raises(RuntimeError):
            accuracy._compute_accuracy(average="unknown average", confusion_matrices=[confusion_matrix])



"""OTX Adapters - mmaction2."""

# Copyright (C) 2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

from .data import OTXActionClsDataset, OTXActionDetDataset
from .models import register_action_backbones
from .utils import Exporter

__all__ = ["OTXActionClsDataset", "OTXActionDetDataset", "Exporter"]

register_action_backbones()
